#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .ParIO import * 
import optparse as op

def init_read_parameters_file(suffix):
    
#    print 'Reading', 'parameters'+suffix
    par = Parameters()
    par.Read_Pars('parameters'+suffix)
    pars = par.pardict

    return pars

def read_ref_values(suffix, pars):

    if 'Bref' in pars:
        Bref = pars['Bref']
    else:
        Bref = 1.
        print( 'Bref not in parameters'+ suffix + \
              '. Bref = 1')

    if 'Tref' in pars:
        Tref = pars['Tref']
    else:
        Tref = 1.
        print( 'Tref not in parameters'+ suffix + \
              '. Tref = 1')

    if 'nref' in pars:
        nref = pars['nref']
    else:
        nref = 1.
        print( 'nref not in parameters'+ suffix + \
              '. nref = 1')

    if 'Lref' in pars:
        Lref = pars['Lref']
    else:
        Lref = 1.
        print( 'Lref not in parameters'+ suffix + \
              '. Lref = 1')

    if 'mref' in pars:
        mref = pars['mref']
    else:
        mref = 1.
        print( 'mref not in parameters'+ suffix + \
              '. mref = 1')
    
    return Bref, Tref, nref, Lref, mref

def read_species_gradients(q_charge, pars):

  if 'x_local' in pars:
      if pars['x_local']:
          x_local = True
      else:
          x_local = False 
  else:
      x_local = True

  if x_local:
    if 'omn1' in pars:
        if pars['charge1'] == q_charge:
            return pars['omn1'], pars['omt1']
        elif 'omn2' in pars:
            if pars['charge2'] == q_charge:
                return pars['omn2'], pars['omt2']
            elif 'omn3' in pars:
                if pars['charge3'] == q_charge:
                    return pars['omn3'], pars['omt3']
                else:
                    return 0, 0
                    print( 'No species with charge = ' + \
                          str(q_charge) + ' is found.' \
                          'omn = 0, omt = 0')
  else:
    return 1.,1.


def read_species_tempdens(q_charge, pars):

    if 'temp1' in pars:
        if pars['charge1'] == q_charge:
            return pars['temp1'], pars['dens1']
        elif 'temp2' in pars:
            if pars['charge2'] == q_charge:
                return pars['temp2'], pars['dens2']
            elif 'temp3' in pars:
                if pars['charge3'] == q_charge:
                    return pars['temp3'], pars['dens3']
                else:
                    return 0, 0
                    print( 'No species with charge = ' + \
                          str(q_charge) + ' iis found.' \
                          'temp = 0, dens = 0')




import numpy as np
from scipy import optimize
import matplotlib.pyplot as plt

#For GUI
import tkinter as tk
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import (FigureCanvasTkAgg,NavigationToolbar2Tk)

#amplitude,mean,stddev=gaussian_fit(x,data,manual=False)
#amplitude,mean,stddev=gaussian_fit(x,data,manual=True)
#amplitude,mean,stddev=gaussian_fit_GUI(root,x,data)

def gaussian(x, amplitude, mean, stddev):
        return amplitude * np.exp(-((x - mean) / (1.*stddev))**2.)

def gaussian_fit(x,data,manual=False,fit_type=0):
    if manual==False:#auto_fit
        if fit_type==0:
            amplitude,mean,stddev=gaussian_fit_auto(x,data)
        elif fit_type==1:
            max_indx=np.argmax(data)
            mean=x[max_indx]
            amplitude=data[max_indx]
            amplitude_sigma=amplitude*np.exp(-1.)
            data_left=data[:max_indx]
            data_right=data[max_indx:]
            x_left=x[:max_indx]
            x_right=x[max_indx:]
            x_left_sigma=x_left[np.argmin(abs(data_left-amplitude_sigma))]
            x_right_sigma=x_right[np.argmin(abs(data_right-amplitude_sigma))]

            stddev=0.5*abs(x_left_sigma-x_right_sigma)
            

            if 1==0:
                print('amplitude,mean,stddev='+str([amplitude,mean,stddev]))
                amplitude_,mean_,stddev_=gaussian_fit_auto(x,data)
                print('amplitude,mean,stddev='+str([amplitude_,mean_,stddev_]))
                popt=[amplitude,mean,stddev]
                popt_=[amplitude_,mean_,stddev_]
                plt.clf()
                plt.plot(x,data)
                plt.axvline(x_left_sigma)
                plt.axvline(x_right_sigma)
                plt.plot(x,gaussian(x, *popt),color='red',label='fit_type 1')
                plt.plot(x,gaussian(x, *popt_),color='green',label='fit_type 0')
                plt.legend()
                plt.show()

    else: #manual_fit using GUI
        amplitude,mean,stddev=gaussian_fit_manual(x,data)
    return amplitude,mean,stddev

def gaussian_fit_GUI(root,x,data):
    x,data=np.array(x), np.array(data)

    global amplitude
    global mean
    global stddev

    top=tk.Toplevel(root)
    top.title('Gaussian Fit')
    #root.geometry("500x500")
    
    #load the icon for the GUI 
    try:
        top.iconbitmap('./SLiM.ico')
    except:
        pass

    #warnings.simplefilter("error", OptimizeWarning)

    #top=tk.LabelFrame(root, text='Gaussian Fit',padx=20,pady=20)
    #top.grid(row=0,column=1)

    frame_plot=tk.LabelFrame(top, text='Plot of the data and fitting',\
                            padx=20,pady=20)
    frame_plot.grid(row=0,column=0)
    fig = Figure(figsize = (5, 5),\
                dpi = 100)
    plot1 = fig.add_subplot(111)
    plot1.plot(x,data, label="data")

    try:
        popt, pcov = optimize.curve_fit(gaussian, x,data)  

        amplitude=popt[0]
        mean=popt[1]
        stddev=popt[2]

        plot1.plot(x, gaussian(x, *popt), label="fit")
        plot1.axvline(mean,color='red',alpha=0.5)
        plot1.axvline(mean+stddev,color='red',alpha=0.5)
        plot1.axvline(mean-stddev,color='red',alpha=0.5)
        plot1.legend()

    except RuntimeError:
        print("Curve fit failed, need to fit manually")
        
        max_index=np.argmax(data)

        amplitude=data[max_index]
        mean=x[max_index]

        data_left=data[:max_index]
        x_left=x[:max_index]
        x_std_index=np.argmin(abs( data_left-amplitude*np.exp(-1.) ))
        stddev=abs(mean-x_left[x_std_index])

        plot1.plot(x, gaussian(x, amplitude, mean, stddev), label="fit")
        plot1.axvline(mean,color='red',alpha=0.5)
        plot1.axvline(mean+stddev,color='red',alpha=0.5)
        plot1.axvline(mean-stddev,color='red',alpha=0.5)
        plot1.legend()

    canvas = FigureCanvasTkAgg(fig,master = frame_plot)
    canvas.draw()
    canvas.get_tk_widget().pack()
    toolbar = NavigationToolbar2Tk(canvas,frame_plot)
    toolbar.update()
    canvas.get_tk_widget().pack()

    root1=tk.LabelFrame(top, text='User Box',padx=20,pady=20)
    root1.grid(row=0,column=1)


    frame1=tk.LabelFrame(root1, text='Accept/Reject the fit',padx=20,pady=20)
    frame1.grid(row=0,column=0)
    #import an option button
    opt_var1= tk.IntVar() #Integar Varible, Other options: StringVar()

    option_button11=tk.Radiobutton(frame1, text='Accept the fit',\
                                variable=opt_var1, value=1,\
                                command=lambda: click_button(opt_var1.get(), top,root1))
    option_button11.grid(row=1,column=0)

    option_button12=tk.Radiobutton(frame1, text='Enter manually',\
                                variable=opt_var1, value=2,\
                                command=lambda: click_button(opt_var1.get(), top,root1))
    option_button12.grid(row=2,column=0)
        

    
    def save_exit(amplitude,mean,stddev):
        looping=0
        top.quit()

    def click_button(value, top, root1):

        global amplitude
        global mean
        global stddev 

        label_list=[]

        frame_data=tk.LabelFrame(root1, text='Current fitting parameter',padx=80,pady=20)
        frame_data.grid(row=1,column=0)
        label_list.append(frame_data)

        amplitude_string=f'amplitude = {amplitude}'
        mean_string     =f'mean      = {mean}'
        std_string      =f'stddev    = {stddev}'
        amplitude_string=amplitude_string+' '*(50-len(amplitude_string))
        mean_string=mean_string+' '*(50-len(mean_string))
        std_string=std_string+' '*(50-len(std_string))

        amp_label =tk.Label(frame_data,text=amplitude_string)
        mean_label=tk.Label(frame_data,text=mean_string)
        std_label =tk.Label(frame_data,text=std_string)
        amp_label.grid(row=0,column=0)
        mean_label.grid(row=1,column=0)
        std_label.grid(row=2,column=0)
        label_list.append(amp_label)
        label_list.append(mean_label)
        label_list.append(std_label)

        def plot_manual_fit(x,data,mean_Input,sigma_Input,top,label_list):
            #frame_plot.grid_forget()

            frame_plot=tk.LabelFrame(top, text='Plot of the data and fitting',padx=20,pady=20)
            frame_plot.grid(row=0,column=0)
            fig = Figure(figsize = (5, 5),
                        dpi = 100)
            plot1 = fig.add_subplot(111)
            plot1.plot(x,data, label="data")
            
            global amplitude
            global mean
            global stddev 

            mean=float(mean_Input)
            amplitude=data[np.argmin(abs(x-mean))]
            stddev=float(sigma_Input)


            frame_data=label_list[0]
            amp_label=label_list[1]
            mean_label=label_list[2]
            std_label=label_list[3]

            amp_label.grid_forget()
            mean_label.grid_forget()
            std_label.grid_forget()

            amplitude_string=f'amplitude = {amplitude}'
            mean_string     =f'mean      = {mean}'
            std_string      =f'stddev    = {stddev}'

            amplitude_string=amplitude_string+' '*(50-len(amplitude_string))
            mean_string=mean_string+' '*(50-len(mean_string))
            std_string=std_string+' '*(50-len(std_string))


            amp_label =tk.Label(frame_data,text=amplitude_string)
            mean_label=tk.Label(frame_data,text=mean_string)
            std_label =tk.Label(frame_data,text=std_string)
            amp_label.grid(row=0,column=0)
            mean_label.grid(row=1,column=0)
            std_label.grid(row=2,column=0)

            plot1.plot(x, gaussian(x, amplitude, mean, stddev), label="fit")
            plot1.axvline(mean,color='red',alpha=0.5)
            plot1.axvline(mean+stddev,color='red',alpha=0.5)
            plot1.axvline(mean-stddev,color='red',alpha=0.5)
            plot1.legend()
        
            canvas = FigureCanvasTkAgg(fig,master = frame_plot)
            canvas.draw()
            canvas.get_tk_widget().pack()
            toolbar = NavigationToolbar2Tk(canvas,frame_plot)
            toolbar.update()
            canvas.get_tk_widget().pack()

        

        if value==1:
            #'Accept the fit'
            myButton2=tk.Button(root1, text='Save and Continue', \
                command=lambda : save_exit(amplitude,mean,stddev))
            myButton2.grid(row=3,column=0)

        elif value==2:
            #Label1.grid_forget()
            #'Enter manually'
            frame_input=tk.LabelFrame(root1, text='Manual Input',padx=10,pady=10)
            frame_input.grid(row=2,column=0)
            tk.Label(frame_input,text='mu(center) = ').grid(row=0,column=0)
            mean_Input_box=tk.Entry(frame_input, width=30, bg='green', fg='white')
            mean_Input_box.insert(0,'')
            mean_Input_box.grid(row=0,column=1)
        
            tk.Label(frame_input,text='sigma(spread) = ').grid(row=1,column=0)
            sigma_Input_box=tk.Entry(frame_input, width=30, bg='green', fg='white')
            sigma_Input_box.insert(0,'')
            sigma_Input_box.grid(row=1,column=1)

            

            Plot_Button=tk.Button(frame_input, text='Plot the Manual Fit',\
                command=lambda: plot_manual_fit(x,data,\
                    float( mean_Input_box.get()  ),\
                    float( sigma_Input_box.get() ),\
                    top,label_list\
                    )  \
                )
            Plot_Button.grid(row=2,column=1)

            Save_Button=tk.Button(root1, text='Save and Continue',\
                     state=tk.DISABLED)#state: tk.DISABLED, or tk.NORMAL
            Save_Button.grid(row=3,column=0)

    try:
        stddev=abs(stddev)
    except:
        amplitude=0.
        mean=0.
        stddev=0.

    print(f'amplitude,mean,stddev={amplitude},{mean},{stddev}')
    top.mainloop()

    
    return amplitude,mean,stddev

def gaussian_fit_auto(x,data):
    judge=0
    try:
        popt, pcov = optimize.curve_fit(gaussian, x,data)  
        #print(gaussian)
        #print(popt)
        #print(pcov)

        max_index=np.argmax(data)
        if 0==1:
            plt.clf()
            plt.plot(x,data, label="data")
            plt.plot(x, gaussian(x, *popt), label="fit")
            plt.axvline(x[max_index],color='red',alpha=0.5)
            plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
            plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
            plt.legend()
            plt.show()

        error_temp=np.sum(abs(data-gaussian(x, *popt)))/abs(np.sum(data))
        #print('norm_error='+str(error_temp))
        if error_temp<0.2:
            amplitude=popt[0]
            mean     =popt[1]
            stddev   =popt[2] 
        else:
            print("Curve fit failed, need to restrict the range")
            amplitude=0
            mean     =0
            stddev   =0
    except RuntimeError:
        print("Curve fit failed, need to restrict the range")
        max_index=np.argmax(data)

        amplitude=data[max_index]
        mean=x[max_index]

        data_left=data[:max_index]
        x_left=x[:max_index]
        x_std_index=np.argmin(abs( data_left-amplitude*np.exp(-1.) ))
        stddev=abs(mean-x_left[x_std_index])
        
    return amplitude,mean,stddev

def gaussian_fit_manual(x,data):
    x,data=np.array(x), np.array(data)

    #warnings.simplefilter("error", OptimizeWarning)
    judge=0

    try:
        popt, pcov = optimize.curve_fit(gaussian, x,data)  
        max_index=np.argmax(data)
        
        
        plt.clf()
        plt.plot(x,data, label="data")
        plt.plot(x, gaussian(x, *popt), label="fit")
        plt.axvline(x[max_index],color='red',alpha=0.5)
        plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
        plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
        plt.legend()
        plt.show()

        judge=int(input("Is the fit okay?  0. No 1. Yes"))

    except RuntimeError:
        print("Curve fit failed, need to restrict the range")

        judge=0
        
        popt=[0]*3  
        max_index=np.argmax(data)
        popt[0]=data[max_index] #amplitud
        popt[1]=x[max_index]    #mean

        plt.clf()
        plt.plot(x,data, label="data")
        plt.axvline(x[max_index],color='red',alpha=0.5)
        plt.legend()
        plt.show()

        while judge==0:

            popt[2]=float(input("sigma="))*np.sqrt(2.)  #stddev

            plt.clf()
            plt.plot(x,data, label="data")
            plt.plot(x, gaussian(x, *popt), label="fit")
            plt.axvline(x[max_index],color='red',alpha=0.5)
            plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
            plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
            plt.legend()
            plt.show()

            judge=int(input("Is the fit okay?  0. No 1. Yes"))


    while judge==0:

        popt=[0]*3  
        max_index=np.argmax(data)
        popt[0]=data[max_index] #amplitud
        popt[1]=x[max_index]    #mean

        popt[2]=float(input("sigma="))*np.sqrt(2.)  #stddev

        plt.clf()
        plt.plot(x,data, label="data")
        plt.plot(x, gaussian(x, *popt), label="fit")
        plt.axvline(x[max_index],color='red',alpha=0.5)
        plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
        plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
        plt.legend()
        plt.show()

        judge=int(input("Is the fit okay?  0. No 1. Yes"))

            
    amplitude=popt[0]
    mean     =popt[1]
    stddev   =popt[2] 

    return amplitude,mean,stddev




from pylab import *
from sys import argv,exit,stdout
import matplotlib.pyplot as plt
from scipy.interpolate import UnivariateSpline as US
from scipy import interpolate
import numpy as np

from .finite_differences import *
from .interp import *

def read_EFIT_file(efit_file_name):

    f = open(efit_file_name,'r')
    eqdsk=f.readlines()
    line1=eqdsk[0].split()
    nw=int(eqdsk[0].split()[-2])
    nh=int(eqdsk[0].split()[-1])
    print ('EFIT file Resolution: %d x %d' %(nw,nh))

    entrylength=16
    #note: here rmin is rleft from EFIT
    try:
        rdim,zdim,rctr,rmin,zmid=[float(eqdsk[1][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[1])/entrylength))]
    except:
        entrylength=15
        try:
            rdim,zdim,rctr,rmin,zmid=[float(eqdsk[1][j*entrylength:(j+1)*entrylength]) for j in range(len(eqdsk[1])/entrylength)]
        except:
            exit('Error reading EQDSK file, please check format!')

    rmag,zmag,psiax,psisep,Bctr=[float(eqdsk[2][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[2])/entrylength))]
    dum,psiax2,dum,rmag2,dum=[float(eqdsk[3][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[3])/entrylength))]
    zmag2,dum,psisep2,dum,dum=[float(eqdsk[4][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[4])/entrylength))]
    if rmag!=rmag2: sys.exit('Inconsistent rmag: %7.4g, %7.4g' %(rmag,rmag2))
    if psiax2!=psiax: sys.exit('Inconsistent psiax: %7.4g, %7.4g' %(psiax,psiax2))
    if zmag!=zmag2: sys.exit('Inconsistent zmag: %7.4g, %7.4g' %(zmag,zmag2) )
    if psisep2!=psisep: sys.exit('Inconsistent psisep: %7.4g, %7.4g' %(psisep,psisep2))

    ###print "rmag", rmag
    ###print "zmag", zmag
    ###print "psiax", psiax
    ###print "psisep", psisep
    ###print "Bctr", Bctr
    # (R,Z) grid on which psi_pol is written
    Rgrid = np.arange(nw)/float(nw-1)*rdim+rmin
    ###print "rdim",rdim
    ###print "rmin",rmin
    ###print "first few Rgrid points", Rgrid[0:6]
    ###print "last few Rgrid points", Rgrid[-7:-1]
    Zgrid = np.arange(nh)/float(nh-1)*zdim+(zmid-zdim/2.0)
    ###print "zdim",zdim
    ###print "zmid",zmid
    ###print "first few Zgrid points", Zgrid[0:6]
    ###print "last few Zgrid points", Zgrid[-7:-1]

    # F, p, ffprime, pprime, q are written on uniform psi_pol grid
    # uniform grid of psi_pol~[psiax,psisep], resolution=nw
    F=empty(nw,dtype=float)
    p=empty(nw,dtype=float)
    ffprime=empty(nw,dtype=float)
    pprime=empty(nw,dtype=float)
    qpsi=empty(nw,dtype=float)
    # psi_pol is written on uniform (R,Z) grid (res=nw(R)*nh(Z))
    psirz_1d=empty(nw*nh,dtype=float)
    
    start_line=5
    lines=range(int(nw/5))
    if nw%5!=0: lines=range(int(nw/5)+1)
    for i in lines:
        n_entries=int(len(eqdsk[i+start_line])/entrylength)
        F[i*5:i*5+n_entries]=[float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) for j in range(n_entries)]
    start_line=i+start_line+1

    for i in lines:
        n_entries=int(len(eqdsk[i+start_line])/entrylength)
        p[i*5:i*5+n_entries]=[float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) for j in range(n_entries)]
    start_line=i+start_line+1

    for i in lines:
        n_entries=int(len(eqdsk[i+start_line])/entrylength)
        ffprime[i*5:i*5+n_entries]=[float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) for j in range(n_entries)]
    start_line=i+start_line+1

    for i in lines:
        n_entries=int(len(eqdsk[i+start_line])/entrylength)
        pprime[i*5:i*5+n_entries]=[float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) for j in range(n_entries)]
    start_line=i+start_line+1

    lines_twod=range(int(nw*nh/5))
    if nw*nh%5!=0: lines_twod=range(int(nw*nh/5)+1)
    for i in lines_twod:
        n_entries=int(len(eqdsk[i+start_line])/entrylength)
        psirz_1d[i*5:i*5+n_entries]=[float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) for j in range(n_entries)]
    start_line=i+start_line+1
    psirz=psirz_1d.reshape(nh,nw)

    for i in lines:
        n_entries=int(len(eqdsk[i+start_line])/entrylength)
        qpsi[i*5:i*5+n_entries]=[float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) for j in range(n_entries)]
    start_line=i+start_line+1

    # even grid of psi_pol, on which all 1D fields are defined
    psip_n = np.linspace(0.0,1.0,nw)
    # return data read from efit file 
    # psip_n: uniform flux grid from magnetic axis to separatrix
    # F, p, ffprime, pprime, qpsi are on psip_n
    # uniform (R,Z) grid, psirz is on this grid
    return psip_n, Rgrid, Zgrid, F, p, ffprime, pprime, psirz, qpsi, rmag, zmag, nw,psiax,psisep

def calc_rho_tor(psip_n, psiax, psisep, qpsi, nw,psip_n_max=0.999):

    #create rho_tor grid on even psi_pol grid
    interpol_order = 3 
    psi_pol = np.empty(len(psip_n))
    for i in range(len(psip_n)):
        psi_pol[i] = psiax+psip_n[i]*(psisep-psiax)

    q_spl_psi = US(psi_pol, qpsi, k=interpol_order, s=1e-5)
    psi_pol_fine = linspace(psi_pol[0], psi_pol[-1], nw*10)
    psi_tor_fine = empty((nw*10),dtype=float)
    psi_tor_fine[0] = 0.
    qfine = np.empty(nw*10)
    ##################
    ##################
    qnew = q_spl_psi(psi_pol_fine)
    psip_n_fine = (psi_pol_fine-psiax)/(psisep-psiax)
    #plt.plot(psip_n_fine,qnew)
    #plt.plot(psip_n,qpsi,'x-')
    #plt.show()
    ##################
    ##################
    for i in range(1,nw*10):
        x=psi_pol_fine[:i+1]
        y=q_spl_psi(x)
        psi_tor_fine[i]=np.trapz(y,x)

    phi_edge = psi_tor_fine[-1]
    rhot_n_fine=np.sqrt(psi_tor_fine/(psi_tor_fine[-1]-psi_tor_fine[0]))
    rho_tor_spl=US(psi_pol_fine, rhot_n_fine, k=interpol_order, s=1e-5)
    # rhot_n grid (not uniform, on even grid of psi_pol) of resolution=nw 
    rhot_n=rho_tor_spl(psi_pol)
    
    # rho_tor_spl takes value of psi_pol (not normalized) and convert into rhot_n
    return rho_tor_spl, rhot_n, phi_edge

def calc_B_fields(Rgrid, rmag, Zgrid, zmag, psirz, psiax, psisep, F, nw, psip_n):
    
    # Z0_ind is the index of Zgrid of midplane
    Z0_ind = np.argmin(np.abs(Zgrid-zmag))
    # psi_midplane is psi_pol at midplane on even Rgrid
    psi_pol_mp = psirz[Z0_ind,:]
    # Rmag_ind is index of unif_R at rmag
    Rmag_ind = np.argmin(np.abs(Rgrid - rmag))
    ###print "rmag",rmag
    ###print "Rmag_ind",Rmag_ind
    ###print "Rgrid[Rmag_ind]~rmag", Rgrid[Rmag_ind]
    ###print "psi_pol_mp[Rmag_ind]~psiax", psi_pol_mp[Rmag_ind]
    psi_pol_obmp = psi_pol_mp[Rmag_ind:].copy()
    #normalize psi_pol_obmp to psip_n_temp
    psip_n_temp = np.empty(len(psi_pol_obmp))
    for i in range(len(psi_pol_obmp)):
        psip_n_temp[i] = (psi_pol_obmp[i]-psiax)/(psisep-psiax)
    unif_R = np.linspace(Rgrid[Rmag_ind],Rgrid[-1],nw*10)
    #unif_R = np.linspace(Rgrid[Rmag_ind],Rgrid[-1],nw)
    psip_n_unifR = interp(Rgrid[Rmag_ind:],psip_n_temp,unif_R)
    psisep_ind = np.argmin(abs(psip_n_unifR-1.02))
    ###print "psisep_ind", psisep_ind
    ###print "psip_n_temp[psisep_ind]~1", psip_n_unifR[psisep_ind]
    #print "we have a problem here because uniform R grid doesn't have enough resolution near separatrix"
    psip_n_obmp = psip_n_unifR[:psisep_ind].copy()
    ###print "psip_n_obmp[0]~0", psip_n_obmp[0]
    ###print "psip_n_obmp[-1]~1", psip_n_obmp[-1]
    #plt.plot(psi_pol_obmp)
    #plt.show()
    R_obmp = unif_R[:psisep_ind].copy()
    # B_pol is d psi_pol/ d R * (1/R)
    #B_pol = fd_d1_o4(psi_pol_obmp, unif_R[Rmag_ind:Rmag_ind+psisep_ind])/unif_R[Rmag_ind:Rmag_ind+psisep_ind]
    B_pol = fd_d1_o4(psip_n_obmp*(psisep-psiax)+psiax,R_obmp)/R_obmp
    # convert F(on even psi_pol grid) to F(on even R grid)
    F_obmp = interp(psip_n, F, psip_n_obmp)
    # B_tor = F/R
    B_tor = F_obmp/R_obmp

    # psip_n_obmp is normalized psi_pol at outboard midplane on uniform unif_R
    # B_tor and B_pol are on uniform unif_R as well
    # psip_n_obmp is unlike psip_n ([0,1]), it goes from 0 to 1.06 here
    return psip_n_obmp, R_obmp, B_pol, B_tor

def read_EFIT_parameters(efit_file_name):

    f = open(efit_file_name,'r')
    eqdsk=f.readlines()
    line1=eqdsk[0].split()
    nw=int(eqdsk[0].split()[-2])
    nh=int(eqdsk[0].split()[-1])

    entrylength=16
    #note: here rmin is rleft from EFIT
    try:
        rdim,zdim,rctr,rmin,zmid=[float(eqdsk[1][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[1])/entrylength))]
    except:
        entrylength=15
        try:
            rdim,zdim,rctr,rmin,zmid=[float(eqdsk[1][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[1])/entrylength))]
        except:
            exit('Error reading EQDSK file, please check format!')

    rmag,zmag,psiax,psisep,Bctr=[float(eqdsk[2][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[2])/entrylength))]
    curr,psiax2,dum,rmag2,dum=[float(eqdsk[3][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[3])/entrylength))]
    zmag2,dum,psisep2,dum,dum=[float(eqdsk[4][j*entrylength:(j+1)*entrylength]) for j in range(int(len(eqdsk[4])/entrylength))] 
    if rmag!=rmag2: sys.exit('Inconsistent rmag: %7.4g, %7.4g' %(rmag,rmag2))
    if psiax2!=psiax: sys.exit('Inconsistent psiax: %7.4g, %7.4g' %(psiax,psiax2))
    if zmag!=zmag2: sys.exit('Inconsistent zmag: %7.4g, %7.4g' %(zmag,zmag2) )
    if psisep2!=psisep: sys.exit('Inconsistent psisep: %7.4g, %7.4g' %(psisep,psisep2))

    return rdim,zdim,rctr,rmin,zmid,Bctr,curr,nh

def get_dimpar_pars(efit_file_name,rhot0):

    psip_n, Rgrid, Zgrid, F, p, ffprime, pprime, psirz, qpsi, rmag, zmag, nw,psiax,psisep = read_EFIT_file(efit_file_name)
    #plt.plot(Rgrid)
    #plt.title('Rgrid (rmag = '+str(rmag)+')')
    #plt.show()
    R_major = rmag
    dummy, rhot_n, phi_edge = calc_rho_tor(psip_n, psiax, psisep, qpsi, nw,psip_n_max=0.999)
    psip_n_obmp, R_obmp, B_pol, B_tor = calc_B_fields(Rgrid, rmag, Zgrid, zmag, psirz, psiax, psisep, F, nw, psip_n)
    Bref = abs(B_tor[0])
    Lref = np.sqrt(2.0*abs(phi_edge/Bref))
    #plt.plot(rhot_n,psip_n)
    #plt.xlabel('rhot_n')
    #plt.ylabel('psi_n')
    #plt.show()
    irhot_n = np.argmin(abs(rhot_n-rhot0))
    q0 = qpsi[irhot_n]
    return Lref, Bref, R_major, q0

def get_current_density(efit_file_name):

    psip_n, Rgrid, Zgrid, F, p, ffprime, pprime, psirz, qpsi, rmag, zmag, nw,psiax,psisep = read_EFIT_file(efit_file_name)
    Jtot = Rgrid*pprime+ffprime/Rgrid
    return psip_n,Rgrid,Jtot


def get_geom_pars(efit_file_name,rhot0):

    psip_n, Rgrid, Zgrid, F, p, ffprime, pprime, psirz, qpsi, rmag, zmag, nw,psiax,psisep = read_EFIT_file(efit_file_name)
    R_major = rmag
    dummy, rhot_n, phi_edge = calc_rho_tor(psip_n, psiax, psisep, qpsi, nw,psip_n_max=0.999)
    psip_n_obmp, R_obmp, B_pol, B_tor = calc_B_fields(Rgrid, rmag, Zgrid, zmag, psirz, psiax, psisep, F, nw, psip_n)
    Bref = abs(B_tor[0])
    Lref = np.sqrt(2.0*abs(phi_edge/Bref))
    irhot_n = np.argmin(abs(rhot_n-rhot0))
    q0 = qpsi[irhot_n]


    rhot_new = np.linspace(rhot_n[0],rhot_n[-1],4*len(rhot_n))
    qpsi_new = interp(rhot_n,qpsi,rhot_new)  
    shat = rhot_new/qpsi_new*fd_d1_o4(qpsi_new,rhot_new)
    irhot_new = np.argmin(abs(rhot_new-rhot0))
    shat0 = shat[irhot_new]
    return Lref, Bref, R_major, q0, shat0






import matplotlib.pyplot as plt
from matplotlib.figure import Figure
import numpy as np
import pandas as pd
from scipy import optimize
import math
import csv
import imageio
from tqdm import tqdm
import sys as sys
sys.path.insert(1, './Tools')
sys.path.insert(1, './SLiM_NN')
from .SLiM_NN.Dispersion_NN import Dispersion_NN
from .Tools.interp import interp
from .Tools.finite_differences import fd_d1_o4
from .Tools.Gaussian_fit import gaussian_fit
from .Tools.Gaussian_fit import gaussian_fit_GUI
from .Tools.max_pedestal_finder import find_pedestal_from_data
from .Tools.max_pedestal_finder import find_pedestal
from .Tools.read_profiles import read_profile_file
from .Tools.read_profiles import read_geom_file
from .Tools.read_EFIT_file import get_geom_pars
from .Tools.write_iterdb import output_iterdb
from .Tools.max_stat_tool import Poly_fit
from .Tools.max_stat_tool import coeff_to_Poly
from .Tools.DispersionRelationDeterminantFullConductivityZeff import VectorFinder_auto
from .Tools.DispersionRelationDeterminantFullConductivityZeff import VectorFinder_manual
from .Tools.DispersionRelationDeterminantFullConductivityZeff import VectorFinder_auto_Extensive
#Created by Max T. Curie  09/17/2021
#Last edited by Max Curie 09/17/2021
#Supported by scripts in IFS


class mode_finder:
    profile_type=('pfile','ITERDB')
    geomfile_type=('gfile','GENE_tracor')
    
    def __init__(self,profile_type,profile_name,geomfile_type,geomfile_name,\
            outputpath,inputpath,x0_min,x0_max,zeff_manual=-1,suffix='.dat',\
            mref=2.,Impurity_charge=6.):
        n0=1.
        Z=float(Impurity_charge)
        #print("suffix"+str(suffix))
        self.profile_name=profile_name
        self.geomfile_name=geomfile_name
        self.outputpath=outputpath
        self.inputpath=inputpath
        self.x0_min=x0_min
        self.x0_max=x0_max

        if profile_type not in mode_finder.profile_type:
            raise ValueError(f'{profile_type} is not a valid profile type, need to be pfile or ITERDB')
        else: 
            self.profile_type  = profile_type

        if geomfile_type not in mode_finder.geomfile_type:
            raise ValueError(f'{geomfile_type} is not a valid geomfile_type, need to be gfile or GENE_tracor')
        else: 
            self.geomfile_type  = geomfile_type

        #*************Loading the data******************************************
        if profile_type=="ITERDB":
            rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0 = read_profile_file(profile_type,profile_name,geomfile_name,suffix)
        elif profile_type=="pfile":
            rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0 = read_profile_file(profile_type,profile_name,geomfile_name,suffix)
    
    
        if geomfile_type=="gfile": 
            xgrid, q, R_ref= read_geom_file(geomfile_type,geomfile_name,suffix)
        elif geomfile_type=="GENE_tracor":
            xgrid, q, Lref, R_ref, Bref, x0_from_para = read_geom_file(geomfile_type,geomfile_name,suffix)
    
        if geomfile_type=="GENE_tracor" and profile_type!="profile":
            rhot0_range_min=np.argmin(abs(rhot0-xgrid[0]))
            rhot0_range_max=np.argmin(abs(rhot0-xgrid[-1]))
            rhot0=rhot0[rhot0_range_min:rhot0_range_max]
            rhop0=rhop0[rhot0_range_min:rhot0_range_max]
            te0=te0[rhot0_range_min:rhot0_range_max]
            ti0=ti0[rhot0_range_min:rhot0_range_max]
            ne0=ne0[rhot0_range_min:rhot0_range_max]
            ni0=ni0[rhot0_range_min:rhot0_range_max]
            vrot0=vrot0[rhot0_range_min:rhot0_range_max]
    
        
        uni_rhot = np.linspace(min(rhot0),max(rhot0),2000)
        
        uni_rhop = interp(rhot0,rhop0,uni_rhot)
        ti_u = interp(rhot0,ti0,uni_rhot)
        ni_u = interp(rhot0,ni0,uni_rhot)
        te_u = interp(rhot0,te0,uni_rhot)
        ne_u = interp(rhot0,ne0,uni_rhot)
        nz_u = interp(rhot0,nz0,uni_rhot)
        vrot_u = interp(rhot0,vrot0,uni_rhot)
        q      = interp(xgrid,q,uni_rhot)
        tprime_e = -fd_d1_o4(te_u,uni_rhot)/te_u
        nprime_e = -fd_d1_o4(ne_u,uni_rhot)/ne_u
        qprime = fd_d1_o4(q,uni_rhot)/q
        
        print(str(x0_min)+', '+str(x0_max))

        midped, topped=find_pedestal(file_name=geomfile_name, path_name='', plot=False)
        x0_center=midped
        print('mid pedestal is at r/a = '+str(x0_center))
    
        if geomfile_type=="gfile": 
            Lref, Bref, R_major, q0, shat0=get_geom_pars(geomfile_name,x0_center)
    
        print("Lref="+str(Lref))
    
        index_begin=np.argmin(abs(uni_rhot-x0_min))
        index_end  =np.argmin(abs(uni_rhot-x0_max))

        ti_u = ti_u[index_begin:index_end]
        te_u = te_u[index_begin:index_end]
        ne_u = ne_u[index_begin:index_end]
        ni_u = ni_u[index_begin:index_end]
        nz_u = nz_u[index_begin:index_end]
        vrot_u = vrot_u[index_begin:index_end]
        q      = q[index_begin:index_end]
        tprime_e = tprime_e[index_begin:index_end]
        nprime_e = nprime_e[index_begin:index_end]
        qprime   = qprime[index_begin:index_end]
        uni_rhot = uni_rhot[index_begin:index_end]

        center_index = np.argmin(abs(uni_rhot-x0_center))
        
        
        #*************End of loading the data******************************************
    
        #****************Start setting up ******************
        ne=ne_u/(10.**19.)      # in 10^19 /m^3
        ni=ni_u/(10.**19.)      # in 10^19 /m^3
        nz=nz_u/(10.**19.)      # in 10^19 /m^3
        te=te_u/1000.           # in keV
        ti=ti_u/1000.           # in keV
        m_SI = mref *1.6726*10**(-27)
        me_SI = 9.11*10**(-31)
        c  = 1.
        qref = 1.6*10**(-19)
        #refes to GENE manual
        coll_c=2.3031*10**(-5)*Lref*ne/(te)**2*(24-np.log(np.sqrt(ne*10**13)/(te*1000)))
        coll_ei=4.*coll_c*np.sqrt(te*1000.*qref/me_SI)/Lref
        nuei=coll_ei
        beta=403.*10**(-5)*ne*te/Bref**2.
    
        nref=ne_u[center_index]
        te_mid=te_u[center_index]
        Tref=te_u[center_index] * qref
        
        cref = np.sqrt(Tref / m_SI)
        Omegaref = qref * Bref / m_SI / c
        rhoref = cref / Omegaref 
        rhoref_temp = rhoref * np.sqrt(te_u/te_mid) 
        kymin=n0*q0*rhoref/(Lref*x0_center)
        kyGENE =kymin * (q/q0) * np.sqrt(te_u/te_mid) * (x0_center/uni_rhot) #Add the effect of the q varying
        #from mtm_doppler
        omMTM = kyGENE*(tprime_e+nprime_e)
        gyroFreq = 9.79E3/np.sqrt(mref)*np.sqrt(te_u)/Lref
        mtmFreq = omMTM*gyroFreq/(2.*np.pi*1000.)
        omegaDoppler = vrot_u*n0/(2.*np.pi*1000.)
        omega=mtmFreq + omegaDoppler
        

        if zeff_manual==-1:
            zeff = (  ( ni+nz*(float(Z)**2.) )/ne  )[center_index]
        else:
            zeff=zeff_manual
        print('********zeff*********')
        print('zeff='+str(zeff))
        print('********zeff*********')
        
        if 1==0:
            plt.clf()
            plt.plot(uni_rhot,omegaDoppler)
            plt.plot(uni_rhot,mtmFreq)
            plt.plot(uni_rhot,omega)
            plt.grid()
            plt.show()

        omega_n_GENE=kyGENE*(nprime_e)       #in cs/a
        omega_n=omega_n_GENE*gyroFreq/(2.*np.pi*1000.)  #in kHz
    
        coll_ei=coll_ei/(2.*np.pi*1000.)  #in kHz

        Lt=1./tprime_e
        Ln=1./nprime_e
        Lq=1./(Lref/(R_ref*q)*qprime)
        
        shat=Ln/Lq
        eta=Ln/Lt
        ky=kyGENE*np.sqrt(2.)
        nu=(coll_ei)/(np.max(omega_n))

        self.mref=mref
        self.Z=Z
        self.Bref=Bref

        self.Lt=Lt
        self.Ln=Ln
        self.Lq=Lq

        self.ne=ne      # in 10^19 /m^3
        self.ni=ni      # in 10^19 /m^3
        self.nz=nz      # in 10^19 /m^3
        self.te=te      # in keV 
        self.ti=ti      # in keV     
        self.nref=nref
        self.r_sigma=(1./shat)*( (me_SI/m_SI)**0.5 )
        self.R_ref=R_ref
        self.cs_to_kHz=gyroFreq/(2.*np.pi*1000.)
        self.omn=omega_n    #omega_n in kHz
        self.cs=cref
        self.rho_s=rhoref*np.sqrt(2.)
        self.Lref=Lref
        self.x=uni_rhot
        self.rhop=uni_rhop
        self.shat=shat
        self.eta=eta
        self.ky=ky
        self.nu=nu
        self.zeff=zeff
        self.beta=beta
        self.q=q
        self.ome=mtmFreq
        self.Doppler=omegaDoppler
        self.coll_ei=coll_ei
        self.gyroFreq=gyroFreq

        self.x_nominal=uni_rhot
        self.nref_nominal=nref
        self.ne_nominal=self.ne     
        self.te_nominal=self.te       
        self.r_sigma_nominal=self.r_sigma
        self.cs_to_kHz_nominal=self.cs_to_kHz
        self.omn_nominal=self.omn
        self.cs_nominal=self.cs
        self.rho_s_nominal=self.rho_s
        self.shat_nominal=self.shat
        self.eta_nominal=self.eta
        self.ky_nominal=self.ky
        self.nu_nominal=self.nu
        self.zeff_nominal=self.zeff
        self.beta_nominal=self.beta
        self.q_nominal=self.q
        self.ome_nominal=self.ome
        self.Doppler_nominal=self.Doppler
        self.coll_ei_nominal=self.coll_ei
        self.gyroFreq_nominal=self.gyroFreq

        self.q_scale=1.
        self.q_shift=0.
        self.shat_scale=1.
        self.ne_scale=1.
        self.ne_shift=0.
        self.te_scale=1.
        self.Doppler_scale=1.


    def __str__(self):
        try:
            self.x_peak
        except:
            self.ome_peak_range(0.1)
        index=np.argmin(abs(self.x-self.x_peak))
        
        return f'Parameter at peak:\n\
        shat={self.shat[index]},\n\
        eta={self.eta[index]},\n\
        ky={self.ky[index]},\n\
        nu={self.nu[index]},\n\
        zeff={self.zeff},\n\
        beta={self.beta[index]},\n\
        r_sigma={self.r_sigma[index]},\n\
        xstar={self.xstar}'


    def set_xstar(self,xstar):
        self.xstar=xstar
        #setter for xstar

    
    def find_pedestal(self,x,p):
        dp=np.gradient(p,x)  #Second order of pressure
        ddp   = np.gradient(dp,x)  #Second order of pressure
        midped = x[np.argmin(dp)]
        topped = x[np.argmin(ddp)]
        return topped,midped

    def q_scale_for_alignment(self,q_at_peak):
        x=self.x
        q=self.q
        ome=self.ome 

        

        index_tmp_=np.argmax(ome)
        q_scale_=q_at_peak/q[index_tmp_]

        if 1==0:
            print(q_at_peak)
            plt.clf()
            plt.plot(x,q)
            plt.plot(x,q*q_scale_)
            plt.axvline(x[index_tmp_])
            plt.show()

        #x2=np.linspace(np.min(x),np.max(x),0.01*abs(x[1]-x[0]))
        #q2=np.interp(x2,x,q)
        #ome2=np.interp(x2,x,ome)

        #index_tmp_=np.argmax(ome2)
        #q_scale_=q_at_peak/q2[index_tmp_]

        return q_scale_

    def inside_freq_band_check(self,f,freq_min_list,freq_max_list):
        for freq_min, freq_max in zip(freq_min_list,freq_max_list):
            if freq_min<=f and f<=freq_max:
                return True
                break


    def profile_fit(self,show_plot=False):
        x=self.x
        ne=self.ne
        te=self.te

        if show_plot==True:
            fig, ax=plt.subplots(nrows=3,ncols=1,sharex=True) 
            ax[0].plot(x,ne,label='original')
            ax[0].set_ylabel(r'$n_e(10^19/m^3)$',fontsize=20)
            ax[1].plot(x,te,label='original')
            ax[1].set_ylabel(r'$T_e(keV)$',fontsize=20)
            ax[1].set_xlabel(r'$\rho_{tor}$',fontsize=20)
            plt.tight_layout()
            plt.show()

        ne_popt, pcov = optimize.curve_fit(tanh_func, x, ne)
        ne_mid_ped, ne_ped_width, ne_height,ne_edge=ne_popt[0], ne_popt[1], ne_popt[2], ne_popt[3]

        te_popt, pcov = optimize.curve_fit(tanh_func, x, te)
        te_mid_ped, te_ped_width, te_height,te_edge=te_popt[0], te_popt[1], te_popt[2], te_popt[3]

        if show_plot==True:
            ne_fit=tanh_func(x, ne_popt[0], ne_popt[1], ne_popt[2], ne_popt[3])
            te_fit=tanh_func(x, te_popt[0], te_popt[1], te_popt[2], te_popt[3])

            fig, ax=plt.subplots(nrows=2,ncols=1,sharex=True) 
            ax[0].plot(x,ne,label='original')
            ax[0].plot(x,ne_fit,label='fit')
            ax[0].legend()
            ax[0].set_ylabel(r'$n_e(10^19/m^3)$',fontsize=10)
            ax[1].plot(x,te,label='original')
            ax[1].plot(x,te_fit,label='fit')
            ax[1].set_ylabel(r'$T_e(keV)$',fontsize=10)
            ax[1].set_xlabel(r'$\rho_{tor}$',fontsize=10)
            plt.tight_layout()
            plt.show()
        
        self.ne_mid_ped=ne_mid_ped
        self.ne_ped_width=ne_ped_width
        self.ne_height=ne_height
        self.ne_edge=ne_edge
        self.te_mid_ped=te_mid_ped
        self.te_ped_width=te_ped_width
        self.te_height=te_height
        self.te_edge=te_edge

        return ne_mid_ped, ne_ped_width, ne_height,ne_edge,\
                te_mid_ped, te_ped_width, te_height,te_edge


    def q_fit(self,q_order=5,show_plot=False):
        dq= np.gradient(self.q,self.x)
        x_dq_min=x[np.argmin(abs(dq))]
        q_coeff=Poly_fit(self.x-x_dq_min, self.q, q_order, show=False)
        q_fit=coeff_to_Poly(self.x, x_dq_min, q_coeff, show=False)
        if show_plot==Ture:
            plt.clf()
            plt.plot(self.x,self.q,label='orginal')
            plt.plot(self.x,q_fit,label='fit')
            plt.legend()
            plt.show()

            
    def modify_profile(self,\
                    q_scale=1.,q_shift=0.,\
                    shat_scale=1.,\
                    ne_scale=1.,te_scale=1.,\
                    ne_shift=0.,te_shift=0.,\
                    Doppler_scale=1.,\
                    show_plot=False):
        
        #************start of modification***********
        #********************************************

        self.q_scale=q_scale
        self.q_shift=q_shift
        self.shat_scale=shat_scale
        self.ne_scale=ne_scale
        self.ne_shift=ne_shift
        self.te_scale=te_scale
        self.Doppler_scale=Doppler_scale

        #**********************************
        #*************modify ne************
        try:
            self.ne_weight
        except:
            ne_top_ped,ne_mid_ped=self.find_pedestal(self.x,self.ne)
            self.ne_mid_ped=ne_mid_ped
            ne_width=ne_mid_ped-ne_top_ped

            self.ne_weight = 0.5+np.tanh((self.x-ne_top_ped)/ne_width)/2

        index=np.argmin(abs(self.x-self.ne_mid_ped))
        ne_mod=self.ne*(1.+(ne_scale-1.)*self.ne_weight)+(ne_shift)*self.ne[index]
        #*************modify ne************
        #**********************************

        #**********************************
        #*************modify Te************
        try:
            self.te_weight
        except:
            te_top_ped,te_mid_ped=self.find_pedestal(self.x,self.te)
            self.te_mid_ped=te_mid_ped
            te_width=te_mid_ped-te_top_ped
            self.te_weight = 0.5+np.tanh((self.x-te_top_ped)/te_width)/2

        if 1==0:
            print('te_top_ped='+str(te_top_ped))
            print('te_mid_ped='+str(te_mid_ped))
            print('te_width='+str(te_width))

        index=np.argmin(abs(self.x-self.te_mid_ped))

        te_mod=self.te*(1.+(te_scale-1.)*self.te_weight)+(te_shift)*self.te[index]
        #*************modify Te************
        #**********************************

        mid_ped,top_ped=self.find_pedestal(self.x,self.ome)


        width=(mid_ped-top_ped)*10.
        try:
            self.shat_weight
        except:
            self.shat_weight = -1.*np.tanh((self.x-mid_ped)/width)/2
        
        q_mod=self.q*q_scale*(1.+(shat_scale-1.)*self.shat_weight)+q_shift

        Doppler_mod=self.Doppler*Doppler_scale

        #show_plot=True

        if show_plot==True:
            fig, ax=plt.subplots(nrows=4,ncols=1,sharex=True) 
            ax[0].plot(self.x,self.ne,label='original')
            ax[0].plot(self.x,ne_mod,label='modified')
            ax[0].legend()
            ax[0].set_ylabel(r'$n_e(10^19/m^3)$',fontsize=10)
            ax[1].plot(self.x,self.te)
            ax[1].plot(self.x,te_mod)
            ax[1].set_ylabel(r'$T_e(keV)$',fontsize=10)
            ax[2].plot(self.x,self.q)
            ax[2].plot(self.x,q_mod)
            ax[2].set_ylabel(r'$q$',fontsize=10)
            ax[3].plot(self.x,self.Doppler)
            ax[3].plot(self.x,Doppler_mod)
            ax[3].set_ylabel(r'$f_{Doppler}(kHz)$',fontsize=10)
            ax[3].set_xlabel(r'$\rho_{tor}$',fontsize=10)
            plt.tight_layout()
            plt.show()

        #*************end of modification************
        #********************************************


        tprime_e = -fd_d1_o4(te_mod,self.x)/te_mod
        nprime_e = -fd_d1_o4(ne_mod,self.x)/ne_mod
        qprime = fd_d1_o4(q_mod,self.x)/q_mod

        center_index=np.argmin(abs(self.x-mid_ped))

        Tref_mod=te_mod[center_index]

        
        #*********start of calculation******
        #****************Start setting up ******************
        #get the data from attribute
        ni=self.ni
        nz=self.nz

        te_u=te_mod*1000.
        ne_u=ne_mod*10.**19.
        ni_u=ni*10.**19.
        nz_u=nz*10.**19.
        mref=self.mref
        Lref=self.Lref
        Bref=self.Bref
        uni_rhot=self.x
        Z=self.Z
        R_ref=self.R_ref


        x0_center=mid_ped
        ne=ne_mod
        te=te_mod
        q=q_mod

        
        n0=1.
        m_SI = mref *1.6726*10**(-27)
        me_SI = 9.11*10**(-31)
        c  = 1.
        qref = 1.6*10**(-19)
        #refes to GENE manual
        coll_c=2.3031*10**(-5)*Lref*ne/(te)**2*(24-np.log(np.sqrt(ne*10**13)/(te*1000)))
        coll_ei=4.*coll_c*np.sqrt(te*1000.*qref/me_SI)/Lref
        nuei=coll_ei
        beta=403.*10**(-5)*ne*te/Bref**2.
    
        
        nref=ne_u[center_index]
        te_mid=te_u[center_index]
        Tref=te_u[center_index]*qref
        q0=q_mod[center_index]
        
        cref = np.sqrt(Tref / m_SI)
        Omegaref = qref * Bref / m_SI / c
        rhoref = cref / Omegaref 
        rhoref_temp = rhoref * np.sqrt(te_u/te_mid) 
        kymin=n0*q0*rhoref/(Lref*x0_center)
        kyGENE =kymin * (q/q0) * np.sqrt(te_u/te_mid) * (x0_center/uni_rhot) #Add the effect of the q varying
        #from mtm_doppler
        omMTM = kyGENE*(tprime_e+nprime_e)
        gyroFreq = 9.79E3/np.sqrt(mref)*np.sqrt(te_u)/Lref
        mtmFreq = omMTM*gyroFreq/(2.*np.pi*1000.)
        omegaDoppler = Doppler_mod
        omega=mtmFreq + omegaDoppler
        
        zeff=self.zeff*nref/self.nref

    
        omega_n_GENE=kyGENE*(nprime_e)       #in cs/a
        omega_n=omega_n_GENE*gyroFreq/(2.*np.pi*1000.)  #in kHz
    
        coll_ei=coll_ei/(2.*np.pi*1000.)  #in kHz

        Lt=1./tprime_e
        Ln=1./nprime_e
        Lq=1./(Lref/(R_ref*q)*qprime)
        
        shat=Ln/Lq
        eta=Ln/Lt
        ky=kyGENE*np.sqrt(2.)
        nu=(coll_ei)/(np.max(omega_n))

        self.x=self.x[2:-2] 
        self.nref=nref
        self.ne=ne[2:-2]      # in 10^19 /m^3
        self.ni=ni[2:-2]      # in 10^19 /m^3
        self.nz=nz[2:-2]      # in 10^19 /m^3
        self.te=te[2:-2]      # in keV      
        self.r_sigma=(1./shat[:-2])*( (me_SI/m_SI)**0.5 )
        self.cs_to_kHz=gyroFreq[:-2]/(2.*np.pi*1000.)
        self.omn=omega_n[2:-2]    #omega_n in kHz
        self.cs=cref
        self.rho_s=rhoref*np.sqrt(2.)
        self.shat=shat[2:-2]
        self.eta=eta[2:-2]
        self.ky=ky[2:-2]
        self.nu=nu[2:-2]
        self.zeff=zeff
        self.beta=beta[2:-2]
        self.q=q[2:-2]
        self.ome=mtmFreq[2:-2]
        self.Doppler=omegaDoppler[2:-2]
        self.coll_ei=coll_ei[2:-2]
        self.gyroFreq=gyroFreq[2:-2]
        
        self.ome_peak_range(0.1)
        #**********end of calculation*******
        if show_plot==True:
            fig, ax=plt.subplots(nrows=7,\
            ncols=1,sharex=True) 
        
            ax[0].plot(self.x_nominal,self.shat_nominal,label='nominal')
            ax[0].plot(self.x,self.shat,label='modified')
            ax[0].legend()
            ax[0].set_ylabel(r'$L_{ne}/L_{q}$')
            ax[1].plot(self.x_nominal,self.eta_nominal)
            ax[1].plot(self.x,self.eta)
            ax[1].set_ylabel(r'$L_{ne}/L_{Te}$')
            ax[2].plot(self.x_nominal,self.ky_nominal)
            ax[2].plot(self.x,self.ky)
            ax[2].set_ylabel(r'$k_y\rho_s$')
            ax[3].plot(self.x_nominal,self.nu_nominal)
            ax[3].plot(self.x,self.nu)
            ax[3].set_ylabel(r'$\nu_{ei}/\omega_{*ne}$')
            ax[4].plot(self.x_nominal,self.beta_nominal)
            ax[4].plot(self.x,self.beta)
            ax[4].set_ylabel(r'$\beta$')
            ax[5].plot(self.x_nominal,self.q_nominal)
            ax[5].plot(self.x,self.q)
            ax[5].set_ylabel(r'$q$')
            ax[6].plot(self.x_nominal,self.ome_nominal)
            ax[6].plot(self.x,self.ome)
            ax[6].set_ylabel(r'$\omega_{*e}(kHz)$')
            ax[6].set_xlabel(r'$\rho_{tor}$')  
            plt.subplots_adjust(wspace=0, hspace=0)
            plt.show()  

    
    def reset_profile(self):
        self.x=self.x_nominal
        self.ne=self.ne_nominal   
        self.te=self.te_nominal
        self.r_sigma=self.r_sigma_nominal
        self.cs_to_kHz=self.cs_to_kHz_nominal
        self.omn=self.omn_nominal
        self.cs=self.cs_nominal
        self.rho_s=self.rho_s_nominal
        self.shat=self.shat_nominal
        self.eta=self.eta_nominal
        self.ky=self.ky_nominal
        self.nu=self.nu_nominal
        self.zeff=self.zeff_nominal
        self.beta=self.beta_nominal
        self.q=self.q_nominal
        self.ome=self.ome_nominal
        self.Doppler=self.Doppler_nominal
        self.coll_ei=self.coll_ei_nominal
        self.gyroFreq=self.gyroFreq_nominal
        self.nref=self.nref_nominal

        self.q_scale=1.
        self.q_shift=0.
        self.shat_scale=1.
        self.ne_scale=1.
        self.ne_shift=0.
        self.te_scale=1.
        self.Doppler_scale=1.


    def q_modify(self,q_scale,q_shift,shat_scale=1.):
        self.q_scale=q_scale
        self.q_shift=q_shift
        self.shat_scale=shat_scale

        mid_ped,top_ped=self.find_pedestal(self.x,self.te)

        width=(mid_ped-top_ped)*10.
        try:
            self.shat_weight
        except:
            self.shat_weight = -1.*np.tanh((self.x-mid_ped)/width)/2

        q_mod=self.q*q_scale*(1.+(shat_scale-1.)*self.shat_weight)+q_shift

        self.ome= self.ome*q_mod/self.q
        self.omn= self.omn*q_mod/self.q
        self.shat= self.shat*( (q_scale*self.q)/q_mod )
        self.ky=self.ky*q_mod/self.q

        self.q=q_mod

        return self.q


    def q_back_to_nominal(self):
        self.q=self.q_nominal
        self.ome= self.ome_nominal
        self.omn= self.omn_nominal
        self.shat= self.shat_nominal
        self.ky=self.ky_nominal

        self.q_scale=1.
        self.q_shift=0.
        self.shat_scale=1.


    def omega_gaussian_fit_GUI(self,root,x,data,rhoref,Lref):
        amplitude,mean,stddev=gaussian_fit_GUI(root,x,data)
        #print(f'amplitude,mean,stddev={amplitude},{mean},{stddev}')
        mean_rho=mean*Lref/rhoref         #normalized to rhoi
        xstar=abs(stddev*Lref/rhoref)
        
        popt=[0]*3
        popt[0] = amplitude
        popt[1] = mean     
        popt[2] = stddev   
    
        #print(popt)
        #print(mean_rho,xstar)
    
        return mean_rho,xstar
  

    def omega_gaussian_fit(self,manual=False,fit_type=0):
        x=self.x 
        data=self.ome 
        rhoref=self.rho_s 
        Lref=self.Lref 

        amplitude,mean,stddev=gaussian_fit(x,data,manual,fit_type)
        #print(f'amplitude,mean,stddev={amplitude},{mean},{stddev}')
        mean_rho=mean*Lref/rhoref         #normalized to rhoi
        xstar=abs(stddev*Lref/rhoref)
        
        popt=[0]*3
        popt[0] = amplitude
        popt[1] = mean     
        popt[2] = stddev   

        #print(popt)
        #print(mean_rho,xstar)
    
        return mean_rho,xstar
  

    def ome_peak_range(self,peak_percent=0.01):
        y1=self.ome
        x=self.x
        peak_index=np.argmax(self.ome)

        left_x=x[:peak_index]
        right_x=x[peak_index:]
        left_y1 =y1[:peak_index]
        right_y1=y1[peak_index:]
    
        left_index=np.argmin(abs(left_y1  - y1[peak_index]*(1.-peak_percent)))
        right_index=np.argmin(abs(right_y1  - y1[peak_index]*(1.-peak_percent)))
        
        
        self.x_peak=self.x[peak_index]
        self.x_min=left_x[left_index]
        self.x_max=right_x[right_index]

        return self.x_peak,self.x_min,self.x_max


    def Rational_surface(self,n0):
        q=self.q
        uni_rhot=self.x

        x_list=[]
        m_list=[]
        
        q_prime=(fd_d1_o4(q,uni_rhot)/q)[1:-1]
        peak_index=np.argmin(abs(q_prime))
        
        qmin = np.min(q)
        qmax = np.max(q)
        m_min = math.ceil(qmin*n0)
        m_max = math.floor(qmax*n0)
        mnums = np.arange(m_min,m_max+1)
        dq=np.max(abs(q[:-1]-q[1:]))
        
        if np.min(q_prime)<-0.001: #for non-monotonic increasing q profiles
            for m in mnums:
                #print(m)
                x_list_temp=[]
                index_list_temp=[]
                q0=float(m)/float(n0)
                for index0 in range(len(q)):
                    if abs(q[index0]-q0)<dq:
                        x_list_temp.append(uni_rhot[index0])
                        index_list_temp.append(index0)
                index_diff=np.array(index_list_temp[:-1],dtype=int)-np.array(index_list_temp[1:],dtype=int)
                #print(index_diff)
                #print(x_list_temp)
                #print(index_list_temp)
                if len(x_list_temp)>0:
                    x_list.append(x_list_temp[0])
                    m_list.append(m)
                for i in range(len(index_diff)):
                    if index_diff[i]==-1:
                        pass 
                    else:
                        x_list.append(x_list_temp[i])
                        m_list.append(m)
    
            #print(x_list)
            #print(m_list)
            return x_list, m_list
        else: #for monotonic increasing q profiles
            for m in mnums:
                q0=float(m)/float(n0)
                index0=np.argmin(abs(q-q0))
                if abs(q[index0]-q0)<0.1:
                    x_list.append(uni_rhot[index0])
                    m_list.append(m)
                
            #print(x_list)
            #print(m_list)
            return x_list, m_list


    def Rational_surface_peak_surface(self,n0):
        try:
            x_peak=self.x_peak
        except:
            x_peak,x_min,x_max=self.ome_peak_range(peak_percent=0.1)

        x_list, m_list=self.Rational_surface(n0)
        index=np.argmin(abs(x_list-x_peak))
        x_surface_near_peak=x_list[index]
        m_surface_near_peak=m_list[index]
        
        return x_surface_near_peak, m_surface_near_peak


    def Rational_surface_top_surfaces(self,n0,top=1):
        try:
            x_peak=self.x_peak
        except:
            x_peak,x_min,x_max=self.ome_peak_range(peak_percent=0.1)

        x_list, m_list=self.Rational_surface(n0)
        temp_ndarray=(np.array([x_list, m_list,abs(x_list-x_peak)])).transpose() 
        surface_df = pd.DataFrame(temp_ndarray,\
                    columns = ['x_list','m_list','peak_distance'])
        #print(surface_df)
        surface_df_sort=surface_df.sort_values(by='peak_distance')

        #print(surface_df_sort)
        #input()
        if len(surface_df_sort['x_list'])<top:
            x_surface_near_peak_list=surface_df_sort['x_list']
            m_surface_near_peak_list=surface_df_sort['m_list']
        else:
            x_surface_near_peak_list=surface_df_sort['x_list'][:top]
            m_surface_near_peak_list=surface_df_sort['m_list'][:top]
        x_surface_near_peak_list=np.array(x_surface_near_peak_list)
        m_surface_near_peak_list=np.array(m_surface_near_peak_list)
        return x_surface_near_peak_list, m_surface_near_peak_list


    def parameter_for_dispersion(self,x0,n):
        index=np.argmin(abs(x0-self.x))

        factor_temp=np.sqrt(\
                (float(n)*self.q[index]/self.Lref)**2.\
                +(float(n)/self.R_ref)**2.)\
                *self.Lref/self.q[index]

        self.factor=factor_temp

        nu=self.nu[index]/factor_temp
        zeff=self.zeff
        eta=self.eta[index]
        shat=self.shat[index]
        beta=self.beta[index]
        ky=self.ky[index]*factor_temp

        try:
            x_peak=self.x_peak
        except:
            x_peak,x_min,x_max=self.ome_peak_range(peak_percent=0.1)

        try:
            xstar=self.xstar
        except:
            xstar=0.
        mu=(x0-x_peak)*self.Lref/(self.rho_s)
        return nu,zeff,eta,shat,beta,ky,mu,xstar


    def Dispersion(self,nu,zeff,eta,shat,beta,ky,ModIndex,mu,xstar,manual=False):
        if manual==True:
            w0=VectorFinder_manual(nu,zeff,eta,shat,beta,ky,ModIndex,abs(mu),xstar)
        elif manual==5:
            w0=VectorFinder_auto_Extensive(nu,zeff,eta,shat,beta,ky,ModIndex,abs(mu),xstar)
        else:
            w0=VectorFinder_auto(nu,zeff,eta,shat,beta,ky,ModIndex,abs(mu),xstar)
        return w0
 

    def plot_parameter(self):
        fig, ax=plt.subplots(nrows=7,\
            ncols=1,sharex=True) 
        
        ax[0].plot(self.x,self.shat)
        ax[0].set_ylabel(r'$L_{ne}/L_{q}$')
        ax[1].plot(self.x,self.eta)
        ax[1].set_ylabel(r'$L_{ne}/L_{Te}$')
        ax[2].plot(self.x,self.ky)
        ax[2].set_ylabel(r'$k_y\rho_s$')
        ax[3].plot(self.x,self.nu)
        ax[3].set_ylabel(r'$\nu_{ei}/\omega_{*ne}$')
        ax[4].plot(self.x,self.beta)
        ax[4].set_ylabel(r'$\beta$')
        ax[5].plot(self.x,self.q,label=r'$q$')
        ax[5].set_ylabel(r'$q$')
        ax[6].plot(self.x,self.ome)
        ax[6].set_ylabel(r'$\omega_{*e}(kHz)$')

        ax[6].set_xlabel(r'$\rho_{tor}$')  
        plt.subplots_adjust(wspace=0, hspace=0)
        plt.show()  


    def Plot_q_scale_rational_surfaces_colored(self,peak_percent,\
            q_scale,q_shift,q_uncertainty,n_list,unstable_list,color_list):
        marker_size=5
        x_peak,x_stability_boundary_min,x_stability_boundary_max=self.ome_peak_range(peak_percent)

        x_min_index=np.argmin(abs(self.x-x_stability_boundary_min))
        x_max_index=np.argmin(abs(self.x-x_stability_boundary_max))
        
        q_range=self.q[x_min_index:x_max_index]

        x_list=[]
        y_list=[]
        x_list_error=[]
        y_list_error=[]
        
        fig, ax = plt.subplots()

        ax.fill_between(self.x, self.q*(1.-q_uncertainty), self.q*(1.+q_uncertainty), color='blue', alpha=.0)
        ax.axvline(x_stability_boundary_min,color='orange',alpha=0.6)
        ax.axvline(x_stability_boundary_max,color='orange',alpha=0.6)
        ax.plot(self.x,self.q,label=r'Safety factor $q_0$')
        
        
        for i in range(len(n_list)):
            n=n_list[i]
            x1=[]
            y1=[]
            x1_error=[]
            y1_error=[]

            qmin = np.min(self.q*(1.-q_uncertainty))
            qmax = np.max(self.q*(1.+q_uncertainty))
        
            m_min = math.ceil(qmin*n)
            m_max = math.floor(qmax*n)
            m_list=np.arange(m_min,m_max+1)

            for m in m_list:
                surface=float(m)/float(n)
                if np.min(q_range)*(1.-q_uncertainty)<surface and surface<np.max(q_range)*(1.+q_uncertainty):
                    x1.append(0.5*(x_stability_boundary_max+x_stability_boundary_min))
                    y1.append(surface)
                    x1_error.append(0.5*(x_stability_boundary_max-x_stability_boundary_min))
                    y1_error.append(0)
            if color_list==-1:
                ax.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',linestyle='none',ms=marker_size,label='n='+str(n))
            else:
                ax.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',color=color_list[i],linestyle='none',ms=marker_size,label='n='+str(n))
            
            x_list.append(x1)
            y_list.append(y1)
            x_list_error.append(x1_error)
            y_list_error.append(y1_error)

        ax.plot(self.x,self.q*q_scale+q_shift,color='orange',label=r'Modified $q=$'+str(q_scale)+r'$*q_0$')
        ax.set_xlim(self.x0_min,self.x0_max)
        ax.set_xlabel(r'$\rho_{tor}$')
        ax.set_ylabel('Safety factor')
        plt.legend(loc='upper left')
        plt.show()
 

    #color_lsit=-1 for auto color assignment
    def Plot_q_scale_rational_surfaces_colored_obj(self,peak_percent,\
            q_scale,q_shift,q_uncertainty,n_list,unstable_list,color_list):
        marker_size=5
        x_peak,x_stability_boundary_min,x_stability_boundary_max=self.ome_peak_range(peak_percent)

        x_min_index=np.argmin(abs(self.x-x_stability_boundary_min))
        x_max_index=np.argmin(abs(self.x-x_stability_boundary_max))
        
        q_range=self.q[x_min_index:x_max_index]

        x_list=[]
        y_list=[]
        x_list_error=[]
        y_list_error=[]
        fig = Figure(figsize = (5, 5),\
                        dpi = 100)
        plot1 = fig.add_subplot(111)
        
        plot1.fill_between(self.x, self.q*(1.-q_uncertainty), self.q*(1.+q_uncertainty), color='blue', alpha=.0)
        plot1.axvline(x_stability_boundary_min,color='orange',alpha=0.6)
        plot1.axvline(x_stability_boundary_max,color='orange',alpha=0.6)
        plot1.plot(self.x,self.q,label=r'Safety factor $q_0$')
        
        
        for i in range(len(n_list)):
            n=n_list[i]
            x1=[]
            y1=[]
            x1_error=[]
            y1_error=[]

            qmin = np.min(self.q*(1.-q_uncertainty))
            qmax = np.max(self.q*(1.+q_uncertainty))
        
            m_min = math.ceil(qmin*n)
            m_max = math.floor(qmax*n)
            m_list=np.arange(m_min,m_max+1)

            for m in m_list:
                surface=float(m)/float(n)
                if np.min(q_range)*(1.-q_uncertainty)<surface and surface<np.max(q_range)*(1.+q_uncertainty):
                    x1.append(0.5*(x_stability_boundary_max+x_stability_boundary_min))
                    y1.append(surface)
                    x1_error.append(0.5*(x_stability_boundary_max-x_stability_boundary_min))
                    y1_error.append(0)
            if color_list==-1:
                plot1.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',linestyle='none',ms=marker_size,label='n='+str(n))
            else:
                plot1.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',color=color_list[i],linestyle='none',ms=marker_size,label='n='+str(n))
            
            x_list.append(x1)
            y_list.append(y1)
            x_list_error.append(x1_error)
            y_list_error.append(y1_error)

        plot1.plot(self.x,self.q*q_scale+q_shift,color='orange',label=r'Modified $q=$'+str(q_scale)+r'$*q_0$')
        plot1.set_xlim(self.x0_min,self.x0_max)
        plot1.set_xlabel(r'$\rho_{tor}$')
        plot1.set_ylabel('Safety factor')
        plot1.legend(loc='upper left')
        

        return fig


    def Plot_q_scale_rational_surfaces_red_and_green(self,peak_percent,\
            q_scale,q_shift,q_uncertainty,n_list,unstable_list):
        marker_size=5
        #calculate the radial stability boundary
        x_peak,x_stability_boundary_min,x_stability_boundary_max=self.ome_peak_range(peak_percent)

        x_min_index=np.argmin(abs(self.x-x_stability_boundary_min))
        x_max_index=np.argmin(abs(self.x-x_stability_boundary_max))

        q_range=self.q[x_min_index:x_max_index]

        x_list=[]
        y_list=[]
        x_list_error=[]
        y_list_error=[]
        fig, ax = plt.subplots()
        ax.fill_between(self.x, self.q*(1.-q_uncertainty), self.q*(1.+q_uncertainty), color='blue', alpha=.2)
        ax.axvline(x_stability_boundary_min,color='orange',alpha=0.6)
        ax.axvline(x_stability_boundary_max,color='orange',alpha=0.6)
        ax.plot(self.x,self.q,color='blue',label=r'Safety factor $q_0$')
        
        stable_counter=0
        unstable_counter=0
        for i in range(len(n_list)):
            n=n_list[i]

            x1=[]
            y1=[]
            x1_error=[]
            y1_error=[]

            qmin = np.min(self.q*(1.-q_uncertainty))
            qmax = np.max(self.q*(1.+q_uncertainty))
        
            m_min = math.ceil(qmin*n)
            m_max = math.floor(qmax*n)
            m_list=np.arange(m_min,m_max+1)

            for m in m_list:
                surface=float(m)/float(n)
                if np.min(q_range)*(1.-q_uncertainty)<surface and surface<np.max(q_range)*(1.+q_uncertainty):
                    x1.append(0.5*(x_stability_boundary_max+x_stability_boundary_min))
                    y1.append(surface)
                    x1_error.append(0.5*(x_stability_boundary_max-x_stability_boundary_min))
                    y1_error.append(0)

            if unstable_list[i]==1:#for unstable case
                if unstable_counter==0:
                    ax.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',linestyle='none',color='red',ms=marker_size,label='Unstable')
                else:
                    ax.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',linestyle='none',color='red',ms=marker_size)
                unstable_counter=unstable_counter+1
            elif unstable_list[i]==0:#for stable case
                if stable_counter==0:
                    ax.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',linestyle='none',color='green',ms=marker_size,label='Stable')
                else:
                    ax.errorbar(x1,y1,xerr=x1_error,yerr=y1_error,marker='s',linestyle='none',color='green',ms=marker_size)
                stable_counter=stable_counter+1
            x_list.append(x1)
            y_list.append(y1)
            x_list_error.append(x1_error)
            y_list_error.append(y1_error)

        ax.plot(self.x,self.q*q_scale+q_shift,color='orange',label=r'Modified $q=$'+str(q_scale)+r'$*q_0$')
        ax.set_xlim(self.x0_min,self.x0_max)
        ax.set_xlabel(r'$\rho_{tor}$')
        ax.set_ylabel('Safety factor')
        plt.legend(loc='upper left')
        plt.show()


    #this attribute only plot the nearest rational surface for the given toroidal mode number
    def Plot_ome_q_surface_demo(self,peak_percent,n_min,n_max,f_min,f_max,with_doppler=False):
        n_list=np.array(np.arange(n_min,n_max+1),dtype=int)
        x_peak,x_min,x_max=self.ome_peak_range(peak_percent)
        x_peak_plot,x_min_plot,x_max_plot=self.ome_peak_range(peak_percent*10.)
        fig, host = plt.subplots()
        fig.subplots_adjust(right=0.75)
        
        #par1 = host.twinx()
        #p1, = par1.plot(self.x, self.q, "b-", label='Safety Factor')
        
        x_fill=[x_min,x_min,x_max,x_max]
        y_fill=[f_min,f_max,f_max,f_min]
        #matplotlib.patches.Rectangle((0,total_trans-trans_error),10,2.*trans_error,alpha=0.4)
        p1, = host.fill(x_fill,y_fill,color='blue',alpha=0.3,label='Unstable area')
        
        
        for n in n_list:
            Unstabel_surface_counter=0
            try:
                x0,m=self.Rational_surface_peak_surface(n)
            except:
                continue

            if x_min_plot<x0 and x0<x_max_plot:
                if x_min<x0 and x0<x_max:
                    host.axvline(x0,color='orange',alpha=1)
                    if with_doppler:
                        host.scatter([x0],[float(n)*(self.ome+self.Doppler)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                    else:
                        host.scatter([x0],[float(n)*(self.ome)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                    Unstabel_surface_counter=Unstabel_surface_counter+1
                else:
                    host.axvline(x0,color='orange',alpha=0.3)
                    if with_doppler:
                        host.scatter([x0],[float(n)*(self.ome+self.Doppler)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                    else:
                        host.scatter([x0],[float(n)*(self.ome)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
            
                    
            if with_doppler:
                if Unstabel_surface_counter>0:
                    p1, = host.plot(self.x,(self.ome+self.Doppler)*float(n), "r-", label=r'Unstable $\omega_{*e}$')
                else:
                    p2, = host.plot(self.x,(self.ome+self.Doppler)*float(n), "k-", label=r'Stable $\omega_{*e}$')
            else:
                if Unstabel_surface_counter>0:
                    p1, = host.plot(self.x,(self.ome)*float(n), "r-", label=r'Unstable $\omega_{*e}$')
                else:
                    p2, = host.plot(self.x,(self.ome)*float(n), "k-", label=r'Stable $\omega_{*e}$')
        

        host.set_xlim(np.min(self.x),np.max(self.x))
        host.set_ylim(0, np.max(f_max)*1.2)
        #par1.set_ylim(np.min(self.q)*0.8,np.max(self.q)*1.2)
        
        host.set_xlabel(r"$\rho_{tor}$")
        host.set_ylabel(r"$\omega_{*e}$(kHz)")
        #par1.set_ylabel("Safety factor")

        
        host.yaxis.label.set_color('black')
        #par1.yaxis.label.set_color('black')
        
        
        tkw = dict(size=4, width=1.5)
        host.tick_params(axis='y', colors='black', **tkw)
        #par1.tick_params(axis='y', colors='black', **tkw)
        #par1.tick_params(axis='y', colors=p3.get_color(), **tkw)
        host.tick_params(axis='x', **tkw)
        
        lines = [p1,p2]
        
        #host.legend(lines, ['Unstable area',r'Unstable $\omega_{*e}$',r'Stable $\omega_{*e}$'])
        host.legend(lines, [l.get_label() for l in lines])
        plt.show()


    def Plot_ome_q_surface_demo_no_box(self,n_min,n_max,n_unstable,with_doppler=False):
        n_list=np.array(np.arange(n_min,n_max+1),dtype=int)
        x_min=np.min(self.x)
        x_max=np.max(self.x)
        fig, host = plt.subplots()
        fig.subplots_adjust(right=0.75)
        
        for n in n_list:
            Unstabel_surface_counter=0
            try:
                x0,m=self.Rational_surface_peak_surface(n)
            except:
                continue

            
            if n in n_unstable:
                host.axvline(x0,color='orange',alpha=1)
                if with_doppler:
                    host.scatter([x0],[float(n)*(self.ome+self.Doppler)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                else:
                    host.scatter([x0],[float(n)*(self.ome)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                Unstabel_surface_counter=Unstabel_surface_counter+1
            else:
                host.axvline(x0,color='orange',alpha=0.3)
                if with_doppler:
                    host.scatter([x0],[float(n)*(self.ome+self.Doppler)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                else:
                    host.scatter([x0],[float(n)*(self.ome)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
            
            if with_doppler:
                if n in n_unstable:
                    p1, = host.plot(self.x,(self.ome+self.Doppler)*float(n), "r-", label=r'Unstable $\omega_{*e}$')
                else:
                    p2, = host.plot(self.x,(self.ome+self.Doppler)*float(n), "k-", label=r'Stable $\omega_{*e}$')
            else:
                if n in n_unstable:
                    p1, = host.plot(self.x,(self.ome)*float(n), "r-", label=r'Unstable $\omega_{*e}$')
                else:
                    p2, = host.plot(self.x,(self.ome)*float(n), "k-", label=r'Stable $\omega_{*e}$')

        
        host.set_xlim(np.min(self.x),np.max(self.x))
        #host.set_ylim(0, np.max(f_max)*1.2)
        #par1.set_ylim(np.min(self.q)*0.8,np.max(self.q)*1.2)
        
        host.set_xlabel(r"$\rho_{tor}$")
        host.set_ylabel(r"$\omega_{*e}$(kHz)")
        #par1.set_ylabel("Safety factor")

        
        host.yaxis.label.set_color('black')
        #par1.yaxis.label.set_color('black')
        
        
        tkw = dict(size=4, width=1.5)
        host.tick_params(axis='y', colors='black', **tkw)
        #par1.tick_params(axis='y', colors='black', **tkw)
        #par1.tick_params(axis='y', colors=p3.get_color(), **tkw)
        host.tick_params(axis='x', **tkw)
        
        try:
            lines = [p1,p2]
        except:
            try:
                lines = [p2]
            except:
                lines = [p1]
        
        #host.legend(lines, ['Unstable area',r'Unstable $\omega_{*e}$',r'Stable $\omega_{*e}$'])
        host.legend(lines, [l.get_label() for l in lines])
        plt.show()


    def Plot_ome_q_surface_frequency_list(self,peak_percent,n_min,n_max,\
                Frequency_list,Frequency_error=0.2,save_imag=False,\
                image_name='./plot.jpg'):
        n_list=np.array(np.arange(n_min,n_max+1),dtype=int)
        x_peak,x_min,x_max=self.ome_peak_range(peak_percent)
        x_peak_plot,x_min_plot,x_max_plot=self.ome_peak_range(peak_percent*10.)

        fig, host = plt.subplots()
        fig.subplots_adjust(right=0.75)
        
        #par1 = host.twinx()
        #p1, = par1.plot(self.x, self.q, "b-", label='Safety Factor')
        for f in Frequency_list:
            f_min=f*(1.-Frequency_error)
            f_max=f*(1.+Frequency_error)
            x_fill=[x_min,x_min,x_max,x_max]
            y_fill=[f_min,f_max,f_max,f_min]
            #matplotlib.patches.Rectangle((0,total_trans-trans_error),10,2.*trans_error,alpha=0.4)
            p1, = host.fill(x_fill,y_fill,color='blue',alpha=0.3,label='Unstable area')
        
        
        for n in n_list:
            Unstabel_surface_counter=0
            try:
                x0,m=self.Rational_surface_peak_surface(n)
            except:
                continue
            if x_min_plot<x0 and x0<x_max_plot:
                if x_min<x0 and x0<x_max:
                    host.axvline(x0,color='orange',alpha=1)
                    host.scatter([x0],[float(n)*(self.ome)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
                    Unstabel_surface_counter=Unstabel_surface_counter+1
                else:
                    host.axvline(x0,color='orange',alpha=0.3)
                    host.scatter([x0],[float(n)*(self.ome)[np.argmin(abs(self.x-x0))]],s=30,color='blue')
            
                    
            if Unstabel_surface_counter>0:
                p2, = host.plot(self.x,(self.ome)*float(n), "r-", label=r'Unstable $\omega_{*e}$')
            else:
                p2, = host.plot(self.x,(self.ome)*float(n), "k-", label=r'Stable $\omega_{*e}$')


        
        f_max=np.max(Frequency_list)
        host.set_xlim(np.min(self.x),np.max(self.x))
        host.set_ylim(0, np.max(f_max)*(1+Frequency_error)*1.2)
        #par1.set_ylim(np.min(self.q)*0.8,np.max(self.q)*1.2)
        
        host.set_xlabel(r"$\rho_{tor}$")
        host.set_ylabel(r"$\omega_{*e}$(kHz)")
        #par1.set_ylabel("Safety factor")

        
        host.yaxis.label.set_color('black')
        #par1.yaxis.label.set_color('black')
        
        
        tkw = dict(size=4, width=1.5)
        host.tick_params(axis='y', colors='black', **tkw)
        #par1.tick_params(axis='y', colors='black', **tkw)
        #par1.tick_params(axis='y', colors=p3.get_color(), **tkw)
        host.tick_params(axis='x', **tkw)
        
        lines = [p1,p2]
        
        #host.legend(lines, ['Unstable area',r'Unstable $\omega_{*e}$',r'Stable $\omega_{*e}$'])
        host.legend(lines, [l.get_label() for l in lines])
        if save_imag==True:
            plt.savefig(image_name)#save the 
        else:
            plt.show()

        plt.close(fig)


    def Plot_ome_q_stability_boundary(self,peak_percent):
        fig, host = plt.subplots()
        fig.subplots_adjust(right=0.75)

        x_peak,x_min,x_max=self.ome_peak_range(peak_percent)
        par1 = host.twinx()
        x=self.x
        y1=self.ome
        y3=self.ome+self.Doppler
        print(self.Doppler)
        y2=self.q
        p1, = host.plot(x, y1, "b-", label=r'$\omega_{*e}$')
        p2, = host.plot(x, y3, "g-", label=r'$\omega_{*e}$ in lab frame')
        p3, = par1.plot(x, y2, "r-", label=r'safety factor')
        p4, = par1.plot([x_min]*1000,np.arange(0,100,(100./1000.)),color='orange',alpha=0.6, label='Unstable range')
        par1.axvline(x_max,color='orange',alpha=0.6)
        
        host.set_xlim(self.x0_min, self.x0_max)
        host.set_ylim(0., np.max(self.ome+self.Doppler)*1.2)
        par1.set_ylim(np.min(self.q)*0.7, np.max(self.q)*1.2)
        
        host.set_xlabel(r"$\rho_{tor}$")
        host.set_ylabel(r"$\omega_{*e}$(kHz)")
        par1.set_ylabel("Safety factor")
        
        
        host.yaxis.label.set_color(p1.get_color())
        par1.yaxis.label.set_color(p3.get_color())
        
        
        tkw = dict(size=4, width=1.5)
        host.tick_params(axis='y', colors=p1.get_color(), **tkw)
        par1.tick_params(axis='y', colors=p3.get_color(), **tkw)
        #par1.tick_params(axis='y', colors=p3.get_color(), **tkw)
        host.tick_params(axis='x', **tkw)
        
        lines = [p1, p2, p3, p4]
        
        host.legend(lines, [l.get_label() for l in lines], loc='upper left')
        plt.show()


    def Plot_highlight_top_percent_ome(self,peak_percent,n_min,n_max,f_min,f_max):
        n_list=np.array(np.arange(n_min,n_max+1),dtype=int)
        x_peak,x_min,x_max=self.ome_peak_range(peak_percent)
        plt.clf()
        fig, host = plt.subplots()
        fig.subplots_adjust(right=0.75)
        fig.set_size_inches(8, 6)
            
        x_min=x_min
        x_max=x_max
        y_min=0
        y_max=10000
        x_fill=[x_min,x_min,x_max,x_max]
        y_fill=[y_min,y_max,y_max,y_min]
        

        p1,=host.plot(self.x,(self.ome+self.Doppler)*float(1), "b-",alpha=0.5, label=r'$\omega_{*e}$')
        p2,=host.fill(x_fill,y_fill,color='purple',alpha=0.1,label=r'$\mu/x_{*} \leq 0.3$')
        
        host.set_xlim(np.min(self.x),np.max(self.x))
        host.set_ylim(0, np.max(self.ome+self.Doppler)*1.2)
        
        host.set_xlabel(r"$\rho_{tor}$")
        host.set_ylabel(r"$\omega_{*e}$(kHz)")
        
        
        host.yaxis.label.set_color('black')
        
        tkw = dict(size=4, width=1.5)
        host.tick_params(axis='y', colors='black', **tkw)
        host.tick_params(axis='x', **tkw)
        
        lines = [ p1,p2]
        
        host.legend(lines, [l.get_label() for l in lines])
        plt.savefig('./define_mu.png')
        plt.show()

    def std_SLiM_calc(self,n_min=1,n_max=8,surface_num=2,Run_mode=6,peak_percent=0.2,manual_fit=False,NN_path='./SLiM_NN/Trained_model'):
        x_list=[]
        n_list=[]
        m_list=[]
        nu_list=[]
        zeff_list=[]
        eta_list=[]
        shat_list=[]
        beta_list=[]
        ky_list=[]
        ModIndex_list=[]
        mu_list=[]
        omega_e_plasma_list=[]
        omega_e_lab_list=[]
        omega_n_kHz_list=[]
        omega_n_cs_a_list=[]
        xstar_list=[]
        q_scale_list0=[]
        q_shift_list0=[]

        self.ome_peak_range(peak_percent)
        mean_rho,xstar=self.omega_gaussian_fit(manual=manual_fit)
        self.set_xstar(xstar)
        
        if Run_mode==1:#simple rational surface alignment
            ModIndex=-1
            filename='./rational_surface_alignment.csv'
        if Run_mode==2 or Run_mode==4 or Run_mode==5 or Run_mode==6:#global dispersion
            ModIndex=1
            filename='./global_dispersion.csv'
        elif Run_mode==3:#local dispersion
            ModIndex=0
            filename='./local_dispersion.csv'
        
        
        peak_index=np.argmin(abs(self.x-self.x_peak))
        omega_e_peak_kHz=self.ome[peak_index]
        
        cs_to_kHz=self.cs_to_kHz[peak_index]
        print('Finding the rational surfaces')
        n0_list=np.arange(n_min,n_max+1,1)

        for n in tqdm(n0_list):
            x_surface_near_peak_list, m_surface_near_peak_list=self.Rational_surface_top_surfaces(n,top=surface_num)
            print(x_surface_near_peak_list)
            print(m_surface_near_peak_list)
            for j in range(len(x_surface_near_peak_list)):
                x_surface_near_peak=x_surface_near_peak_list[j]
                m_surface_near_peak=m_surface_near_peak_list[j]
                if self.x_min<=x_surface_near_peak and x_surface_near_peak<=self.x_max:
                    nu,zeff,eta,shat,beta,ky,mu,xstar=\
                        self.parameter_for_dispersion(x_surface_near_peak,n)
                    factor=self.factor
                    index=np.argmin(abs(self.x-x_surface_near_peak))
                    omega_n_kHz=float(n)*self.omn[index]
                    omega_n_cs_a=float(n)*self.omn[index]/cs_to_kHz
                    omega_e_plasma_kHz=float(n)*self.ome[index]
                    omega_e_lab_kHz=float(n)*self.ome[index]+float(n)*self.Doppler[index]
                
                    n_list.append(n)
                    m_list.append(m_surface_near_peak)
                    x_list.append(x_surface_near_peak)
                    nu_list.append(nu)
                    zeff_list.append(zeff)
                    eta_list.append(eta)
                    shat_list.append(shat)
                    beta_list.append(beta)
                    ky_list.append(ky)
                    ModIndex_list.append(ModIndex)
                    mu_list.append(mu)
                    xstar_list.append(xstar)
                    omega_e_plasma_list.append(omega_e_plasma_kHz)
                    omega_e_lab_list.append(omega_e_lab_kHz)
                    omega_n_kHz_list.append(omega_n_kHz)
                    omega_n_cs_a_list.append(omega_n_cs_a)
                    q_scale_list0.append(self.q_scale)
                    q_shift_list0.append(self.q_shift)
        
        
        d = {'q_scale':q_scale_list0,'q_shift':q_shift_list0,\
            'n':n_list,'m':m_list,'rho_tor':x_list,\
            'omega_plasma_kHz':[0]*len(n_list),\
            'omega_lan_kHz':[0]*len(n_list),\
            'gamma_cs_a':[0]*len(n_list),\
            'omega_n_kHz':omega_n_kHz_list,\
            'omega_n_cs_a':omega_n_cs_a_list,\
            'omega_e_plasma_kHz':omega_e_plasma_list,\
            'omega_e_lab_kHz':omega_e_lab_list,\
            'peak_percentage':omega_e_plasma_list/\
                    (omega_e_peak_kHz*np.array(n_list,dtype=float)),\
            'nu':nu_list,'zeff':[zeff]*len(n_list),'eta':eta_list,\
            'shat':shat_list,'beta':beta_list,'ky':ky_list,\
            'ModIndex':ModIndex_list,'mu':mu_list,'xstar':xstar_list}
        df=pd.DataFrame(d)   #construct the panda dataframe
    
        if Run_mode==6:
            print('**************')
            print('**************')
            print(Run_mode)
            if hasattr(self, 'Dispersion_NN_obj'):
                pass
            else:
                self.Dispersion_NN_obj=Dispersion_NN(NN_path)

        
        if Run_mode==1:
            df_calc=0
            df_para=df
        else:    
            with open(filename, 'w', newline='') as csvfile:     #clear all and then write a row
                data = csv.writer(csvfile, delimiter=',')
                data.writerow(['q_scale','q_shift','n','m','rho_tor',\
                    'omega_plasma_kHz','omega_lab_kHz',\
                    'gamma_cs_a','omega_n_kHz',\
                    'omega_n_cs_a','omega_e_plasma_kHz',\
                    'omega_e_lab_kHz','peak_percentage',\
                    'nu','zeff','eta','shat','beta','ky',\
                    'ModIndex','mu','xstar'])
            csvfile.close()

            print('Calculate the dispersion relations')
            
            for i in tqdm(range(len(df['nu']))):
                if Run_mode==4:
                    w0=self.Dispersion(df['nu'][i],df['zeff'][i],df['eta'][i],\
                        df['shat'][i],df['beta'][i],df['ky'][i],\
                        df['ModIndex'][i],df['mu'][i],df['xstar'][i],manual=True)
                elif Run_mode==5:
                    w0=self.Dispersion(df['nu'][i],df['zeff'][i],df['eta'][i],\
                        df['shat'][i],df['beta'][i],df['ky'][i],\
                        df['ModIndex'][i],df['mu'][i],df['xstar'][i],manual=5)
                elif Run_mode==6:
                    w0=self.Dispersion_NN_obj.Dispersion_omega(df['nu'][i],df['zeff'][i],df['eta'][i],\
                            df['shat'][i],df['beta'][i],df['ky'][i],df['mu'][i],df['xstar'][i])
                else:
                    w0=self.Dispersion(df['nu'][i],df['zeff'][i],df['eta'][i],\
                        df['shat'][i],df['beta'][i],df['ky'][i],\
                        df['ModIndex'][i],df['mu'][i],df['xstar'][i])
                
                omega=np.real(w0)
                omega_kHz=omega*omega_n_kHz_list[i]
                gamma=np.imag(w0)
                gamma_cs_a=gamma*omega_n_cs_a_list[i]

                with open(filename, 'a+', newline='') as csvfile: #adding a row
                    data = csv.writer(csvfile, delimiter=',')
                    data.writerow([ q_scale_list0[i],\
                        q_shift_list0[i],\
                        df['n'][i],df['m'][i],df['rho_tor'][i],\
                        omega_kHz,\
                        omega_kHz+df['omega_e_lab_kHz'][i]-df['omega_e_plasma_kHz'][i],\
                        gamma_cs_a,\
                        df['omega_n_kHz'][i],df['omega_n_cs_a'][i],\
                        df['omega_e_plasma_kHz'][i],df['omega_e_lab_kHz'][i],
                        df['peak_percentage'][i],df['nu'][i],\
                        df['zeff'][i],df['eta'][i],\
                        df['shat'][i],df['beta'][i],df['ky'][i],\
                        df['ModIndex'][i],df['mu'][i],df['xstar'][i] ])
                csvfile.close()
            df_calc=pd.read_csv(filename)
            df_para=df
        return df_calc,df_para




#refered from: https://youtu.be/tEFkHEKypLI

#step1: python setup.py sdist bdist_wheel
#step2: twine upload dist/*

from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.39'
DESCRIPTION = 'SLiM project and its supporting scripts'
LONG_DESCRIPTION = 'SLiM project calculate the slab like microtearing mode with global effect consideration'

# Setting up
setup(
    name="SLiM_phys",
    version=VERSION,
    author="maxtcurie (Max Curie)",
    author_email="<maxtcurie@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=[],
    #install_requires=['scipy', 'numpy', 'pandas',\
    #                'matplotlib', 'tqdm','sys',\
    #                'csv','math','os','cmath',\
    #                'tkinter'],
    keywords=['python', 'plasma', 'physics', \
            'microtearing modes', 'reduced model', \
            'neural network'],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    Homepage = "https://github.com/maxtcurie/SLiM/wiki"
)



#This script is intended to find the top and the mid pedestal of the H mod plasma profile for the pre and post processing of the simulation

#Developed by Max Curie on 02/03/2020

import numpy as np
import matplotlib.pyplot as plt
import scipy.optimize as opt
import re
from .max_stat_tool import *
# some_file.py
import sys


#The command of this script will be "python max_parity_calculator.py 0001(# of the scan)" 

def parity_finder_long(zgrid,f,name,plot,report): #this function is for local linear simulation with long range of z 

    f=np.real(f)
    #For the case the zgrid is evenly distributed. 
    zmin=np.min(zgrid)
    zmax=np.max(zgrid)
    nz=len(zgrid)
    #print(zmax)
    #print(zmin)

    z0=int(nz/(zmax-zmin)) #nz from zmin to zmax
    parity_name=['even','odd']
    location=[-1,0,1]
    parity=np.zeros((len(location)+1,2))
    
    parity_plot_abs0=[]
    parity_plot_abs1=[]
    for j in range(len(location)):
        for i in range(z0):
            ztemp=int(nz/2+location[j]*z0) #around -1,0,1
            #even parity sum(f(x)-f(-x)) = 0 if even 
            parity[j][0]=parity[j][0]+abs(f[ztemp+i-1]-f[ztemp-i])
            #odd parity sum(f(x)+f(-x)) = 0 if odd 
            parity[j][1]=parity[j][1]+abs(f[ztemp+i-1]+f[ztemp-i])


    #********Doing the same thing for loop***********
    z_loop, f_loop=loop(zgrid,f,-1,1)
    for i in range(z0):
        ztemp=z0
        parity_plot_abs0.append(abs(f_loop[ztemp+i-1]-f_loop[ztemp-i]))
        parity_plot_abs1.append(abs(f_loop[ztemp+i-1]+f_loop[ztemp-i]))
        #even parity sum(f(x)-f(-x)) = 0 if even 
        ntemp=len(location)
        parity[ntemp][0]=parity[ntemp][0]+abs(f_loop[ztemp+i-1]-f_loop[ztemp-i])
        #odd parity sum(f(x)+f(-x)) = 0 if odd 
        parity[ntemp][1]=parity[ntemp][1]+abs(f_loop[ztemp+i-1]+f_loop[ztemp-i])
    #***********Find the ratio at different locations****************
    ratio=np.zeros((len(location)+1)*2)
    for i in range(len(location)+1):
        ratio[2*i+0]=parity[i][0]/(parity[i][0]+parity[i][1]) #percentage of oddness
        ratio[2*i+1]=parity[i][1]/(parity[i][0]+parity[i][1]) #percentage of evenness
    
    #ratio[2*0+0,2*0+1]=-1    {percentage of oddness, percentage of eveness}
    #ratio[2*1+0,2*1+1]= 0    {percentage of oddness, percentage of eveness}
    #ratio[2*2+0,2*2+1]= 1    {percentage of oddness, percentage of eveness}
    #ratio[2*3+0,2*3+1]= loop {percentage of oddness, percentage of eveness}
    location0=location[int(np.argmin(ratio[0:len(location)*2-1])/2)]
    parity0=parity_name[int(np.argmin(ratio[0:len(location)*2-1])%2)]


    #********calc the ratio for loop**********************
   
    if parity[3][1]<parity[3][0]:
        parity1='odd'
    else:
        parity1='even'

    #*********Print out report*****************
    if report==1:
        for i in range(len(location)+1):
            if i < len(location):
                print('Around z=',location[i])
            else:
                print('For the loop')        
            print(ratio[2*i+0]*100,'% Odd', ratio[2*i+1]*100, "% Even")

        print('The location of the center is', location0)
        print('The function is largely', parity0)

        print('Based on the loop, the function is largely ',parity1)
    
        if parity1==parity0:
            print('Result checked')
        else:
            print('location=', location0 ,'and loop mismatch, please check the Parity_plot.png or function.png to determine the parity manually')
    #*********End of Print out report**********

    #********Plot the result*******************
    if plot ==1:
        x_zoom,y_zoom = zoom1D(zgrid,f,-2,2)
        plt.clf()
        plt.title('Parity of'+ name+ 'calculation')
        plt.xlabel(r'$z/\pi$',fontsize=10)
        plt.ylabel(r'$f$',fontsize=13)
        #plt.plot(np.arange(0,1,1/len(parity_plot0)),parity_plot_abs0,label="Even function abs")
        #plt.plot(np.arange(0,1,1/len(parity_plot0)),parity_plot_abs1,label="Odd function abs")
        plt.plot(x_zoom,y_zoom,label=name)
        plt.plot(z_loop,f_loop,label=name+'_loop')
        plt.axhline(y=0, color="red")
        plt.axvline(x=location0, label='Center of the symmetry', color="red")
        #plt.axvline(x=midped, label="Mid-pedestal", color="red")
        #lt.axvline(x=topped, label="Top-pedestal", color="green")
        plt.legend()
        plt.savefig('Parity_plot_'+ name +'.png')
        plt.show()

        plt.clf()
        plt.title(name)
        plt.xlabel(r'$z/\pi$',fontsize=10)
        plt.ylabel(r'$f$',fontsize=13)
        #plt.plot(z_loop,f_loop,label="loop_f")
        plt.plot(zgrid,f,label=name)
        #plt.axvline(x=midped, label="Mid-pedestal", color="red")
        #lt.axvline(x=topped, label="Top-pedestal", color="green")
        plt.legend()
        plt.savefig('function_'+name+'.png')
        plt.show()
        
        ntemp=int(np.argmin(ratio[0:len(location)*2-1])/2)
        ratio0=np.zeros(2)
        ratio0[0]=ratio[ntemp*2+0]
        ratio0[1]=ratio[ntemp*2+1]

    return parity0,location0,ratio0

def parity_finder_short(zgrid,f,name,plot,report): #this function is for local linear simulation with short range of z 

    f=np.real(f)
    #For the case the zgrid is evenly distributed. 
    zmin=np.min(zgrid)
    zmax=np.max(zgrid)

    if abs(zmin) < abs(zmax):
        idx=find_nearest_index(zgrid,-zmax)
        new_z=zgrid[idx:-1]
        f=f[idx:-1]
    elif abs(zmin) > abs(zmax):
        idx=find_nearest_index(zgrid,-zmin)
        new_z=zgrid[0:idx]
        f=f[0:idx]
    else:
        new_z=zgrid
        f=f

    idx=find_nearest_index(new_z,0)
    #print(f'index of z=0 is {idx}')
    #print(f'index of nz/2 is {int(len(new_z)/2)}')

    z0=int(len(new_z)/2)

    location0=0
    parity=np.zeros(2)


    for i in range(z0):
        ztemp=int(z0) #around -1,0,1
        #even parity sum(f(x)-f(-x)) = 0 if even 
        parity[0]=parity[0]+abs(f[ztemp+i-1]-f[ztemp-i])
        #odd parity sum(f(x)+f(-x)) = 0 if odd 
        parity[1]=parity[1]+abs(f[ztemp+i-1]+f[ztemp-i])

    #Determine function's parity
    if parity[1]<parity[0]:
        parity1='odd'
    else:
        parity1='even'

    if plot ==1:
        plt.clf()
        plt.title('Parity of'+name+' calculation')
        plt.xlabel(r'$z/\pi$',fontsize=10)
        plt.ylabel(name,fontsize=13)
        #plt.plot(np.arange(0,1,1/len(parity_plot0)),parity_plot_abs0,label="Even function abs")
        #plt.plot(np.arange(0,1,1/len(parity_plot0)),parity_plot_abs1,label="Odd function abs")
        plt.plot(new_z,f,label=name)
        plt.axhline(y=0, color="red")
        plt.axvline(x=location0, label='Center of the symmetry', color="red")
        #plt.axvline(x=midped, label="Mid-pedestal", color="red")
        #lt.axvline(x=topped, label="Top-pedestal", color="green")
        plt.legend()
        plt.savefig('Parity_plot_'+name+'.png')
        plt.show()

    ratio=np.zeros(2)
    ratio[0]=parity[0]/(parity[0]+parity[1]) #percentage of oddness
    ratio[1]=parity[1]/(parity[0]+parity[1]) #percentage of evenness

    return parity1,location0,ratio

def parity_finder_short2(zgrid,f): #this function is for local linear simulation with short range of z 
    len_z_half=int(len(zgrid)/2)
    f_inv=np.flip(f) 
    #even parity sum(f(x)-f(-x)) = 0 if even -- f(x)= f(-x)
    evenness=np.sum(abs(f[len_z_half:]-f_inv[len_z_half:]))
    #odd  parity sum(f(x)+f(-x)) = 0 if odd  -- f(x)=-f(-x)
    oddness=np.sum(abs(f[len_z_half:]+f_inv[len_z_half:]))
    total=evenness+oddness
    oddness_norm=1.-oddness/total #percentage of oddness
    eveness_norm=1.-evenness/total #percentage of evenness

    return oddness_norm,eveness_norm


def parity_finder_general(zgrid,f,name,plot,report):
    zmin=np.min(zgrid)
    zmax=np.max(zgrid)
    if zmax>=2 and zmin<= -2: 
        return parity_finder_long(zgrid,f,name,plot,report)
    else:
        return parity_finder_short(zgrid,f,name,plot,report)
'''
x=np.arange(-6,6,0.01)
test_function1=np.sin(x)
test_function2=np.cos(x)
test_function3=x*np.cos(x)
oddness_norm,eveness_norm=parity_finder_short2(x,test_function1)
print([oddness_norm,eveness_norm])
'''


def read_gfile_to_dict(g_file_name):
    f=open(g_file_name,'r')
    data = f.read()
    f.close()

    sdata = data.split('\n')

    date = sdata[0].split()[1]
    discharge_num = int(sdata[0].split()[2][1:])
    time = int(sdata[0].split()[3][:-2])
    n0 = int(sdata[0].split()[4])
    nx = int(sdata[0].split()[5])
    ny = int(sdata[0].split()[6]) 

    print()

    g_quant_dict['q']={'header':''}

    return g_quant_dict



import optparse as op
import math
import cmath
import sys
import numpy as np
from scipy import optimize
import matplotlib.pyplot as plt

#********************************************************
#***********Define the function**************************
#Function average:
#Use the weighted average to "smooth" out the bump and rescaling
def smooth(avg_list,bin_size):
    l_list=len(avg_list)
    list_avg=np.zeros(l_list-bin_size+1)
    list_dev=np.zeros(l_list-bin_size+1)
    avg=0
    for i in range(0,len(list_avg)):
      list_avg[i]=np.mean(avg_list[i:i+bin_size]) 
      list_dev[i]=np.std(avg_list[i:i+bin_size]) 

    return list_avg, list_dev


def avg(avg_list,bin_size):
#weighted average check chapter 7 in an introduction to error analysisby John R. Taylor
  l_list=len(avg_list)
  #print avg_list
  #print 'l_list', l_list
  avg=0
  dev_sum=0
  #print 'Check point', l_list-bin_size+1
  for i in range(0,l_list-bin_size+1):
    #print i
    avg_temp=np.mean(avg_list[i:i+bin_size])
    std_temp=np.std(avg_list[i:i+bin_size])
    #print avg_temp
    #print std_temp
    avg=avg+avg_temp/(std_temp)**2
    dev_sum=dev_sum+1/(std_temp)**2
  if dev_sum == 0: 
    avg=avg
    dev_sum=10**20
  else:
    avg=avg/dev_sum #average
    dev_sum=np.sqrt(1/dev_sum)
  
  output=np.zeros(2)
  output[0]=avg
  output[1]=dev_sum
  return(output)

def avg_dev(avg_list,dev_list):
  bin_size=3 #bin size
  l_list=len(avg_list)
  avg=0
  dev=0
  dev_sum=0
  for i in range(0,l_list-bin_size+1):
    avg_temp=np.mean(avg_list[i:i+bin_size])
    dev_temp=np.mean(dev_list[i:i+bin_size])
    std_temp=np.std(avg_list[i:i+bin_size])
    avg=avg+avg_temp/(std_temp)**2
    dev=dev+dev_temp/(std_temp)**2
    dev_sum=dev_sum+1/(std_temp)**2
  avg=avg/dev_sum #average
  dev=dev/dev_sum #standard dev
  output=np.zeros(2)
  output[0]=avg
  output[1]=dev
  return(output)

def norm(a_list): #Normalized to the 1
  return(a_list/np.max(abs(a_list)))


#def step(list,center,): 

def zoom1D(x,y,x_min,x_max):
#this function zoon in the a 1D plot
  x_zoom=[]
  y_zoom=[]
  for i in range(len(x)):
    if x[i]<=x_max and x[i]>=x_min:
      x_zoom.append(x[i])
      y_zoom.append(y[i])
  return x_zoom, y_zoom

def zoom2D(x,y,z,x_min,x_max,y_min,y_max):
#this function zoon in the a 1D plot
  x_zoom=[]
  y_zoom=[]
  z_zoom=[]
  for i in range(len(x)):
    if x[i]<=x_max and x[i]>=x_min:
      if y[i]<=y_max and y[i]>=y_min:
        x_zoom.append(x[i])
        y_zoom.append(y[i])
        z_zoom.append(z[i])
  return x_zoom, y_zoom, z_zoom

#this function is from unutbu https://stackoverflow.com/questions/2566412/find-nearest-value-in-numpy-array
def find_nearest_index(array, value): #this function return the index of the a value(or nearest)
  array = np.asarray(array)
  idx = (np.abs(array - value)).argmin()
  return idx

def find_nearest(array, value): #this function return the index of the a value(or nearest)
  array = np.asarray(array)
  idx = (np.abs(array - value)).argmin()
  return array[idx]


def loop(x,f,x_min,x_max):#this function make the function return [sum f(x+n*x0)] where x0 is the period
  x0=x_max-x_min

  x = np.asarray(x)#same as find_nearest_index
  nx_min = (np.abs(x - x_min)).argmin()#same as find_nearest_index
  nx_max = (np.abs(x - x_max)).argmin()#same as find_nearest_index

  x_loop=x[nx_min:nx_max]
  f_loop=np.zeros(len(x_loop))

  x_loop = np.asarray(x_loop)
  for i in range(len(x)):
    xtemp=(x[i]-x_min)%x0+x_min
    nxtemp = np.argmin(np.abs(x_loop - xtemp)) #same as find_nearest_index
    f_loop[nxtemp]=f_loop[nxtemp]+f[i]

  return x_loop, f_loop


#x_sort,f_sort = sort_x_f(x,f)
def sort_x_f(x_unsort,f_unsort): 
   
    arr_unsort=[x_unsort,f_unsort]
    f_x_unsort=tuple(map(tuple, np.transpose(arr_unsort)))
      
    f_x_sort=sorted(f_x_unsort, key=lambda f_x_unsort: f_x_unsort[0])
    f_x_sort=np.array(f_x_sort)
    f_x_sort=np.transpose(f_x_sort)
    x_sort=f_x_sort[0,:]
    f_sort=f_x_sort[1,:]

    return x_sort,f_sort


def gaussian(x, amplitude, mean, stddev):
    return amplitude * np.exp(-0.5*((x - mean) /stddev)**2)

def gaussian_fit(x,data):
    x,data=np.array(x), np.array(data)

    #warnings.simplefilter("error", OptimizeWarning)
    judge=0

    try:
        popt, pcov = optimize.curve_fit(gaussian, x,data)  

        max_index=np.argmax(data)
        
        plt.clf()
        plt.plot(x,data, label="data")
        plt.plot(x, gaussian(x, *popt), label="fit")
        plt.axvline(x[max_index],color='red',alpha=0.5)
        plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
        plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
        plt.legend()
        plt.show()

        judge=int(input("Is the fit okay?  0. No 1. Yes"))

    except RuntimeError:
        print("Curve fit failed, need to restrict the range")

        judge=0
        
        popt=[0]*3  
        max_index=np.argmax(data)
        popt[0]=data[max_index] #amplitud
        popt[1]=x[max_index]    #mean

        plt.clf()
        plt.plot(x,data, label="data")
        plt.axvline(x[max_index],color='red',alpha=0.5)
        plt.legend()
        plt.show()

        while judge==0:

            popt[2]=float(input("sigma="))  #stddev

            plt.clf()
            plt.plot(x,data, label="data")
            plt.plot(x, gaussian(x, *popt), label="fit")
            plt.axvline(x[max_index],color='red',alpha=0.5)
            plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
            plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
            plt.legend()
            plt.show()

            judge=int(input("Is the fit okay?  0. No 1. Yes"))


    while judge==0:

        popt=[0]*3  
        max_index=np.argmax(data)
        popt[0]=data[max_index] #amplitud
        popt[1]=x[max_index]    #mean

        popt[2]=float(input("sigma="))  #stddev

        plt.clf()
        plt.plot(x,data, label="data")
        plt.plot(x, gaussian(x, *popt), label="fit")
        plt.axvline(x[max_index],color='red',alpha=0.5)
        plt.axvline(x[max_index]+popt[2],color='red',alpha=0.5)
        plt.axvline(x[max_index]-popt[2],color='red',alpha=0.5)
        plt.legend()
        plt.show()

        judge=int(input("Is the fit okay?  0. No 1. Yes"))

            
    amplitude=popt[0]
    mean     =abs(popt[1])
    stddev   =abs(popt[2])

    return amplitude,mean,stddev

# p=np.polyfit(uni_rhot,q,5)

def Poly_fit(x, data, order, show):
    """ Returns a polynomial for ``x`` values for the ``coeffs`` provided.

    The coefficients must be in ascending order (``x**0`` to ``x**o``).
    """

    #https://numpy.org/doc/stable/reference/generated/numpy.polyfit.html
    coeffs=np.polyfit(x,data,order)
    coeffs=np.flip(coeffs)

    o = len(coeffs)
    print(f'# This is a polynomial of order {ord}.')
    y = 0
    for i in range(o):
        y += coeffs[i]*x**i

    if show==True:
        plt.clf()
        plt.plot(x,y,label='fit')
        plt.plot(x,data,label='original')
        plt.legend()
        plt.show()

    return coeffs
def coeff_to_Poly(x, x0, coeff, show):

    y = 0
    for i in range(len(coeff)):
        y += coeff[i]*(x-x0)**i

    if show==True:
        plt.clf()
        plt.plot(x,y)
        plt.show()

    return y